#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import site
import os
import sys


# Get directory in which packages are installed
try:
    packagedir = site.getsitepackages()[0]
except AttributeError:
    # Inside virtualenvironment, so getsitepackages doesnt work.
    paths = sys.path
    for p in paths:
        if os.path.isdir(p) and os.path.basename(p) == 'site-packages':
            packagedir = p

# Add the WORC FASTR tools and type paths
tools_path = [os.path.join(packagedir, 'WORC', 'resources', 'fastr_tools')] + tools_path
types_path = [os.path.join(packagedir, 'WORC', 'resources', 'fastr_types')] + types_path

# Mounts accessible to fastr virtual file system
mounts['worc_example_data'] = os.path.join(packagedir, 'WORC', 'exampledata')
mounts['apps'] = os.path.expanduser(os.path.join('~', 'apps'))
mounts['output'] = os.path.expanduser(os.path.join('~', 'WORC', 'output'))
mounts['test'] = os.path.join(packagedir, 'WORC', 'resources', 'fastr_tests')

# The ITKFile type requires a preferred type when no specification is given.
# We will set it to Nifti, but you may change this.
preferred_types += ["NiftiImageFileCompressed"]



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

IS_TEST = True


def create_network():
    # Import the faster environment and set it up
    import fastr

    # Create a new network
    network = fastr.Network(id_='CalcFeatures_test')

    # Create a source node in the network
    source_segmentation = network.create_source('ITKImageFile', id_='segmentation')
    source_image = network.create_source('ITKImageFile', id_='image')
    source_metadata = network.create_source('DicomImageFile', id_='metadata')
    source_parameters = network.create_source('ParameterFile', id_='parameters')

    # Create a new node in the network using toollist
    node_calfeatures = network.create_node('CalcFeatures', id_="calcfeatures")

    # Create a link between the source output and an input of the addint node
    node_calfeatures.inputs['segmentation'] = source_segmentation.output
    node_calfeatures.inputs['image'] = source_image.output
    node_calfeatures.inputs['metadata'] = source_metadata.output
    node_calfeatures.inputs['parameters'] = source_parameters.output

    # Create a sink to save the data
    sink_features = network.create_sink('HDF5', id_='features')

    # Link the addint node to the sink
    sink_features.input = node_calfeatures.outputs['features']

    return network


def source_data(network):
    return {'segmentation': 'vfs://worc_example_data/CLM/seg_tumor_c.nii.gz',
            'image':  'vfs://worc_example_data/CLM/image.nii.gz',
            'metadata':  'vfs://worc_example_data/CLM/00000.dcm',
            'parameters': 'vfs://worc_example_data/CLM/parameters.ini'}


def sink_data(network):
    return {'features': 'vfs://tmp/results/{}/calcfeatures_{{sample_id}}_{{cardinality}}{{ext}}'.format(network.id)}


def main():
    network = create_network()

    # Execute
    # network.draw_network()
    network.execute(source_data(network), sink_data(network), execution_plugin="ProcessPoolExecution")


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

IS_TEST = True


def create_network():
    # Import the faster environment and set it up
    import fastr

    network = fastr.Network(id_="elastix_test")

    source1 = network.create_source('ITKImageFile', id_='fixed_img')
    source2 = network.create_source('ITKImageFile', id_='moving_img')
    param1 = network.create_source('ElastixParameterFile', id_='param_file')

    elastix_node = network.create_node('elastix_dev', id_='elastix')
    elastix_node.inputs['fixed_image'] = source1.output
    elastix_node.inputs['moving_image'] = source2.output
    link_param = network.create_link(param1.output, elastix_node.inputs['parameters'])
    link_param.converge = 0

    outtrans = network.create_sink('ElastixTransformFile', id_='sink_trans')
    outtrans.inputs['input'] = elastix_node.outputs['transform']

    transformix_node = network.create_node('transformix_dev', id_='transformix')
    transformix_node.inputs['image'] = source2.output
    transformix_node.inputs['transform'] = elastix_node.outputs['transform'][-1]

    outimage = network.create_sink('ITKImageFile', id_='sink_image')
    outimage.inputs['input'] = transformix_node.outputs['image']

    network.draw_network(img_format='svg')
    network.dumpf('{}.json'.format(network.id), indent=2)

    return network


def source_data(network):
    return {'fixed_img': {'s1': 'vfs://worc_example_data/elastix/img0/slice047.mhd'},
            'moving_img': {'s1': 'vfs://worc_example_data/elastix/img1/slice091.mhd'},
            'param_file': ['vfs://worc_example_data/elastix/parAslice.txt', 'vfs://worc_example_data/elastix/parBslice.txt']}


def sink_data(network):
    return {'sink_trans': 'vfs://tmp/results/{}/elastix_output_trans_{{sample_id}}_{{cardinality}}{{ext}}'.format(network.id),
            'sink_image': 'vfs://tmp/results/{}/elastix_output_image_{{sample_id}}_{{cardinality}}{{ext}}'.format(network.id)}


def main():
    network = create_network()

    # Execute
    # network.draw_network()
    network.execute(source_data(network), sink_data(network), execution_plugin="ProcessPoolExecution")


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pandas as pd
import json
import csv
import argparse
from natsort import natsorted
import glob
import os


def main():
    parser = argparse.ArgumentParser(description='Radiomics results')
    parser.add_argument('-svm', '--svm', metavar='svm',
                        nargs='+', dest='svm', type=str, required=True,
                        help='SVM file (HDF)')
    parser.add_argument('-perf', '--perf', metavar='Performance', nargs='+',
                        dest='perf', type=str, required=True,
                        help='Performance (Parameters + Statistics)')
    parser.add_argument('-ts', '--ts', metavar='ts', nargs='+',
                        dest='ts', type=str, required=False,
                        help='Total Sensitivity from PCE Analysis (.csv)')
    parser.add_argument('-res', '--res', metavar='Results', nargs='+',
                        dest='res', type=str, required=True,
                        help='Results')
    args = parser.parse_args()

    if type(args.svm) is list:
        args.svm = ''.join(args.svm)

    if type(args.ts) is list:
        args.ts = ''.join(args.ts)

    if type(args.perf) is list:
        args.perf = ''.join(args.perf)

    # If input is dir, use glob
    print args.svm
    if os.path.isdir(args.svm):
        args.svm = glob.glob(args.svm + '/svm_*.hdf5')
        args.svm = natsorted(args.svm)

    if os.path.isdir(args.perf):
        args.perf = glob.glob(args.perf + '/performance_*.json')
        args.perf = natsorted(args.perf)

    if args.ts is not None:
        if os.path.isdir(args.ts):
            args.ts = glob.glob(args.ts + '/TS_*.csv')
            args.ts = natsorted(args.ts)
    else:
        args.ts = [None] * len(args.svm)

    # Remove provenance
    temp = list()
    for pf in args.perf:
        if 'prov' not in pf:
            temp.append(pf)

    args.perf = temp

    # Read the inputs: labels of SVM, parameters, sensitivity
    feature_labels = list()
    # TODO: Fix "Too Few Features" SVMs
    all_feature_labels = list()
    pftemp = list()
    tstemp = list()
    for svmfile, tsfile, pffile in zip(args.svm, args.ts, args.perf):
        svm = pd.read_hdf(svmfile)
        if 'Too Few Features.' in svm.keys() or ' Too Few Features.' in svm.keys():
            print(("Too few features in {}.").format(svmfile))
        else:
            print svm
            labels = svm[svm.keys()[0]].ix['feature_labels'] # .tolist()
            feature_labels.append(labels)
            all_feature_labels = all_feature_labels + list(set(labels) - set(all_feature_labels))
            pftemp.append(pffile)
            tstemp.append(tsfile)

    args.perf = pftemp
    args.ts = tstemp
    parameters = list()
    statistics = list()
    alwaysright = list()
    alwayswrong = list()
    ar_labels = list()
    aw_labels = list()
    for parafile in args.perf:
        with open(parafile) as fp:
            parameters_temp = json.load(fp)

        parameters.append(parameters_temp['Parameters'])
        statistics.append(parameters_temp['Statistics'])

        # Extract always right and wrong labels
        ar_temp = parameters_temp['Statistics']['Always right']
        aw_temp = parameters_temp['Statistics']['Always wrong']

        ar = dict()
        aw = dict()
        for k in ar_temp.keys():
            ar[k + ' (Right)'] = ar_temp[k]
            if (k + ' (Right)') not in ar_labels:
                ar_labels.append(k + ' (Right)')

        for k in aw_temp.keys():
            aw[k + ' (Wrong)'] = aw_temp[k]
            if (k + ' (Wrong)') not in aw_labels:
                aw_labels.append(k + ' (Wrong)')

        # Set patient ID labels
        alwaysright.append(ar)
        alwayswrong.append(aw)

    parameter_labels = parameters[0].keys()
    statistics_labels = statistics[0].keys()

    # Fill in the missing patient labels

    # Read in total sensitivity
    if args.ts[0] is not None:
        ts = list()
        for tsfile in args.ts:
            ts_temp = list()
            with open(tsfile, 'rb') as f:
                reader = csv.reader(f)
                for row in reader:
                    ts_temp.append(row[0])
            ts.append(ts_temp)

        # Combine the svm labels with the ts values in one dictionary
        sensitivity = list()
        for label, values in zip(feature_labels, ts):
            sensitivity_temp = dict()
            for key, item in zip(label, values):
                sensitivity_temp[key] = item
            sensitivity.append(sensitivity_temp)

    else:
        sensitivity = [None]

    # Join all labels and values
    # if type(patient_labels) == dict:
    #     patient_labels = patient_labels.keys()
    #
    # patient_labels_right = list()
    # patient_labels_wrong = list()
    # for k in patient_labels:
    #     patient_labels_right.append(k + ' (Right)')
    #     patient_labels_wrong.append(k + ' (Wrong)')

    if sensitivity[0] is not None:
        all_labels = parameter_labels + statistics_labels +\
            natsorted(ar_labels) + natsorted(aw_labels) +\
            natsorted(all_feature_labels)

        parameter_grid = list()
        for para, stat, sens, right, wrong in zip(parameters, statistics,
                                                  sensitivity, alwaysright,
                                                  alwayswrong):
            z = dict()
            for d in [para, stat, sens, right, wrong]:
                for k, v in d.iteritems():
                    z[k] = v

            # Fill in missing values in dict with blanks
            for key in all_labels:
                if key not in z.keys():
                    z[key] = ''

            parameter_grid.append(z)
    else:
        all_labels = parameter_labels + statistics_labels +\
            natsorted(ar_labels) + natsorted(aw_labels)

        parameter_grid = list()
        for para, stat, right, wrong in zip(parameters, statistics,
                                                  alwaysright,
                                                  alwayswrong):
            z = dict()
            for d in [para, stat, right, wrong]:
                for k, v in d.iteritems():
                    z[k] = v

            # Fill in missing values in dict with blanks
            for key in all_labels:
                if key not in z.keys():
                    z[key] = ''

            parameter_grid.append(z)

    # Writing to csv file
    if type(args.res) == list:
        args.res = ''.join(args.res)

    with open(args.res, 'w') as f:
        writer = csv.DictWriter(f, fieldnames=all_labels)
        writer.writeheader()
        for parameter_point in parameter_grid:
            writer.writerow(parameter_point)

    print("Done writing results!")


if __name__ == '__main__':
    main()


# 'python' '/scratch/mstarmans/WORC/fastr_Tools/worc/bin/combineresults.py' '--svm' 'svm_13.hdf5' 'svm_1.hdf5' 'svm_12.hdf5' 'svm_11.hdf5' 'svm_18.hdf5' 'svm_15.hdf5' 'svm_10.hdf5' 'svm_14.hdf5' 'svm_0.hdf5' 'svm_19.hdf5' 'svm_17.hdf5' 'svm_5.hdf5' 'svm_2.hdf5' 'svm_4.hdf5' 'svm_7.hdf5' 'svm_6.hdf5' 'svm_3.hdf5' 'svm_9.hdf5' 'svm_8.hdf5' '--perf' 'performance_13.json' 'performance_1.json' 'performance_12.json' 'performance_11.json' 'performance_18.json' 'performance_15.json' 'performance_10.json' 'performance_14.json' 'performance_0.json' 'performance_19.json' 'performance_17.json' 'performance_5.json' 'performance_2.json' 'performance_4.json' 'performance_7.json' 'performance_6.json' 'performance_3.json' 'performance_9.json' 'performance_8.json' '--ts' 'TS_13.csv' 'TS_1.csv' 'TS_12.csv' 'TS_11.csv' 'TS_18.csv' 'TS_15.csv' 'TS_0.csv' 'TS_14.csv' 'TS_10.csv' 'TS_19.csv' 'TS_17.csv' 'TS_5.csv' 'TS_2.csv' 'TS_4.csv' 'TS_7.csv' 'TS_6.csv' 'TS_3.csv' 'TS_9.csv' 'TS_8.csv' '--res' 'results.csv'

# 'python' '/scratch/mstarmans/WORC/fastr_Tools/worc/bin/combineresults.py' '--svm' 'svm_13.hdf5' 'svm_1.hdf5' 'svm_12.hdf5' 'svm_11.hdf5' 'svm_15.hdf5' 'svm_10.hdf5' 'svm_14.hdf5' 'svm_0.hdf5' 'svm_19.hdf5' 'svm_17.hdf5' 'svm_5.hdf5' 'svm_2.hdf5' 'svm_4.hdf5' 'svm_7.hdf5' 'svm_6.hdf5' 'svm_3.hdf5' 'svm_9.hdf5' 'svm_8.hdf5' '--perf' 'performance_13.json' 'performance_1.json' 'performance_12.json' 'performance_11.json' 'performance_15.json' 'performance_10.json' 'performance_14.json' 'performance_0.json' 'performance_19.json' 'performance_17.json' 'performance_5.json' 'performance_2.json' 'performance_4.json' 'performance_7.json' 'performance_6.json' 'performance_3.json' 'performance_9.json' 'performance_8.json' '--ts' 'TS_13.csv' 'TS_1.csv' 'TS_12.csv' 'TS_11.csv' 'TS_15.csv' 'TS_0.csv' 'TS_14.csv' 'TS_10.csv' 'TS_19.csv' 'TS_17.csv' 'TS_5.csv' 'TS_2.csv' 'TS_4.csv' 'TS_7.csv' 'TS_6.csv' 'TS_3.csv' 'TS_9.csv' 'TS_8.csv' '--res' 'results.csv'

# 'python' '/home/mstarmans/WORC/fastr_Tools/worc/bin/combineresults.py' '--svm' '/archive/mstarmans/Output/CLM_0503_M1_GL' '--perf' '/archive/mstarmans/Output/CLM_0503_M1_GL' '--ts' '/archive/mstarmans/Output/CLM_0503_M1_GL' '--res' 'results_CLM_0503F1_GL.csv'



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import SimpleITK as sitk
import fastr
import numpy as np
import os
import WORC.addexceptions as WORCexceptions

#WIP

class Transformix(object):
    def __init__(self):
        self.network = self.create_network()
        self.MovingImage = None
        self.TransformParameterMap = None

    def create_network(self):
        self.network = fastr.Network(id_="transformix")

        self.MovingImageSource = self.network.create_source('ITKImageFile', id_='MovingImage')
        self.ParameterMapSource = self.network.create_source('ElastixTransformFile', id_='ParameterFile')

        self.transformix_node = self.network.create_node('transformix_dev', id_='transformix')
        self.transformix_node.inputs['image'] = self.MovingImageSource.output
        self.transformix_node.inputs['transform'] = self.ParameterMapSource.output

        self.outimage = self.network.create_sink('ITKImageFile', id_='sink_image')
        self.outimage.inputs['input'] = self.transformix_node.outputs['image']

        self.network.draw_network(img_format='svg')
        self.network.dumpf('{}.json'.format(self.network.id), indent=2)

    def execute(self):
        SimpleTransformix = sitk.SimpleTransformix()



#!/usr/bin/env python

# Copyright 2011-2017 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
from PREDICT.trainclassifier import trainclassifier


def main():
    parser = argparse.ArgumentParser(description='Radiomics classification')
    parser.add_argument('-feat_m1', '--feat_m1', metavar='features_m1',
                        nargs='+', dest='feat_m1', type=str, required=True,
                        help='Patient features input of first modality (HDF)')
    parser.add_argument('-feat_m2', '--feat_m2', metavar='features_m2',
                        nargs='+', dest='feat_m2', type=str, required=False,
                        default=None,
                        help='Patient features input of second modality (HDF)')
    parser.add_argument('-feat_m3', '--feat_m3', metavar='features_m3',
                        nargs='+', dest='feat_m3', type=str, required=False,
                        default=None,
                        help='Patient features input of third modality (HDF)')
    parser.add_argument('-pc', '--pc', metavar='Patientinfo', dest='pc',
                        type=str, required=True, nargs='+',
                        help='Classification of patient')
    parser.add_argument('-cf', '--conf', metavar='config', nargs='+',
                        dest='cf', type=str, required=True,
                        help='Configuration')
    parser.add_argument('-c', '--class', metavar='classification',
                        dest='svm', type=str, required=True, nargs='+',
                        help='Classification (HDF)')
    parser.add_argument('-perf', '--perf', metavar='performance',
                        dest='perf', type=str, required=True, nargs='+',
                        help='Performance (JSON)')
    parser.add_argument('-fs', '--fs', metavar='fixedsplits',
                        dest='fs', type=str, required=False, nargs='+',
                        help='File containing fixed splits for iterations (XLSX)')
    args = parser.parse_args()

    trainclassifier(feat_m1=args.feat_m1, feat_m2=args.feat_m2,
                    feat_m3=args.feat_m3,
                    config=args.cf, patientinfo=args.pc,
                    output_svm=args.svm, output_json=args.perf, verbose=False,
                    fixedsplits=args.fs)


if __name__ == '__main__':
    main()



# Copyright 2011-2014 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
from fastr.datatypes import URLType


class ElastixTransformFile(URLType):
    _re_line_parse = re.compile(r'\s*(\(.+\s+.*\))?\s*(//.*)?')
    _re_keyvalue_parse = re.compile(r'\((\S+)\s+(.*)\)')
    _re_value_parse = re.compile(r'(".*?"|-?\d+\.?\d*)')

    description = 'Elastix Transform parameter file'
    extension = 'txt'

    def __eq__(self, other):
        if type(other) is not ElastixTransformFile:
            return NotImplemented

        return self._parse() == other._parse()

    def _parse(self, filename=None):
        data = dict()

        if filename is None:
            filename = self.parsed_value

        with open(filename, 'r') as input_file:
            for linenr, line in enumerate(input_file):
                match_obj = re.match(self._re_line_parse, line)

                if match_obj.group(1) is not None:
                    submatch = re.match(self._re_keyvalue_parse, match_obj.group(1))
                    key = submatch.group(1)
                    value = re.findall(self._re_value_parse, submatch.group(2))

                data[key] = tuple(self._parse_value(x) for x in value)

        initial_field = 'InitialTransformParametersFileName'
        if initial_field in data:
            initial_filename = data[initial_field][0]

            if initial_filename != 'NoInitialTransform':
                if initial_filename[0] not in '\\/':
                    initial_filename = os.path.join(os.path.dirname(filename), initial_filename)
                data[initial_field] = self._parse(initial_filename)

        return data

    def _parse_value(self, item):
        if item is None:
            return None

        if isinstance(item, (int, float)):
            return item

        if item[0] == '"' and item[-1] == '"':
            item = item[1:-1]
        else:
            try:
                item = int(item)
            except ValueError:
                try:
                    item = float(item)
                except ValueError:
                    item = str(item)

        return item



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module contains all WORC-related Exceptions
"""

# import inspect
# import os
# import textwrap

# pylint: disable=too-many-ancestors
# Because fo inheriting from FastrError and a common exception causes this
# exception, even though this behaviour is desired


class WORCError(Exception):
    """
    This is the base class for all WORC related exceptions. Catching this
    class of exceptions should ensure a proper execution of WORC.
    """
    # def __init__(self, *args, **kwargs):
    #     """
    #     Constructor for all exceptions. Saves the caller object fullid (if
    #     found) and the file, function and line number where the object was
    #     created.
    #     """
    #     super(WORCError, self).__init__(*args, **kwargs)
    #
    #     frame = inspect.stack()[1][0]
    #     call_object = frame.f_locals.get('self', None)
    #     if call_object is not None and hasattr(call_object, 'fullid'):
    #         self.WORC_object = call_object.fullid
    #     else:
    #         self.WORC_object = None
    #
    #     info = inspect.getframeinfo(frame)
    #     self.filename = info.filename
    #     self.function = info.function
    #     self.linenumber = info.lineno
    #
    # def __str__(self):
    #     """
    #     String representation of the error
    #
    #     :return: error string
    #     :rtype: str
    #     """
    #     if self.WORC_object is not None:
    #         return '[{}] {}'.format(self.WORC_object, super(WORCError, self).__str__())
    #     else:
    #         return super(WORCError, self).__str__()
    #
    # def excerpt(self):
    #     """
    #     Return a excerpt of the Error as a tuple.
    #     """
    #     return type(self).__name__, self.message, self.filename, self.linenumber
    pass


class WORCNotImplementedError(WORCError, NotImplementedError):
    """
    This function/method has not been implemented on purpose (e.g. should be
    overwritten in a sub-class)
    """
    pass


class WORCIOError(WORCError, IOError):
    """
    IOError in WORC
    """
    pass


class WORCTypeError(WORCError, TypeError):
    """
    TypeError in the WORC system
    """
    pass



#!/usr/bin/env python

# Copyright 2011-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
from WORC.processing.RTStructReader import RTStructReader


def main():
    parser = argparse.ArgumentParser(description='RTStruct Reader')
    parser.add_argument('-rt', '--rt', metavar='rt', nargs='+',
                        dest='rt', type=str, required=False,
                        help='RT Struct file to read (DICOM)')
    parser.add_argument('-roiname', '--roiname', metavar='roiname',
                        dest='roiname',
                        type=str, required=False, nargs='+',
                        help='Name of ROI to extract (string)')
    parser.add_argument('-out', '--out', metavar='out',
                        dest='out', type=str, required=False,
                        help='Image output (ITK Image)')
    args = parser.parse_args()

    if type(args.rt) is list:
        args.rt = ''.join(args.rt)

    if type(args.roiname) is list:
        args.roiname = ''.join(args.roiname)

    if type(args.out) is list:
        args.out = ''.join(args.out)

    RTStructReader(args.rt, args.roiname)


if __name__ == '__main__':
    main()



# Copyright 2011-2014 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from fastr.datatypes import URLType


class TransformixPointFile(URLType):
    description = 'Text file to store point coordinates'
    extension = 'txt'

    def _validate(self):
        # Special case, for no points, but entire field
        if not super(TransformixPointFile, self)._validate():
            return False

        try:
            # Extract lines
            lines = []
            with open(self.parsed_value) as fin:
                for line in fin:
                    lines.append(line)

            # Check first line is index or point
            if lines[0].strip() not in ['index', 'point']:
                return False

            # Check second line is number of points
            try:
                nrpoints = int(lines[1])
            except ValueError:
                return False

            # Check number of points in file
            if len(lines) != nrpoints + 2:
                return False
        except IOError:
            return False

        return True



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PREDICT.plotting.plot_ROC import plot_ROC
import argparse


def main():
    parser = argparse.ArgumentParser(description='Plot the ROC Curve of an estimator')
    parser.add_argument('-prediction', '--prediction', metavar='prediction',
                        nargs='+', dest='prediction', type=str, required=True,
                        help='Prediction file (HDF)')
    parser.add_argument('-pinfo', '--pinfo', metavar='pinfo',
                        nargs='+', dest='pinfo', type=str, required=True,
                        help='Patient Info File (txt)')
    parser.add_argument('-ensemble', '--ensemble', metavar='ensemble',
                        nargs='+', dest='ensemble', type=str, required=True,
                        help='Length of ensemble (int)')
    parser.add_argument('-label_type', '--label_type', metavar='label_type',
                        nargs='+', dest='label_type', type=str, required=True,
                        help='Label name that is predicted (string)')
    parser.add_argument('-output_png', '--output_png', metavar='output_png',
                        nargs='+', dest='output_png', type=str, required=False,
                        help='File to write output to (PNG)')
    parser.add_argument('-output_csv', '--output_csv', metavar='output_csv',
                        nargs='+', dest='output_csv', type=str, required=False,
                        help='File to write output to (csv)')
    parser.add_argument('-output_tex', '--output_tex', metavar='output_tex',
                        nargs='+', dest='output_tex', type=str, required=False,
                        help='File to write output to (tex)')
    args = parser.parse_args()

    plot_ROC(prediction=args.prediction,
             pinfo=args.pinfo,
             ensemble=args.ensemble,
             label_type=args.label_type,
             output_png=args.output_png,
             output_tex=args.output_tex,
             output_csv=args.output_csv)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import argparse
import os
import subprocess
from elastixparameterfile import ElastixParameterFile


def create_def_transform(input_transformation, output_transformation, deformation_file=None, verbose=False):
    """
    Create a deformation transformation from any other transformation. This
    will call elastix to create the deformation field and creates a new
    transformation file based on the input transform.

    :param str input_transformation: path of the input transformation
    :param str output_Trasnformation: path to write the output transformation
    :param bool verbose: flag to control verbosity
    """
    output_dir = os.path.dirname(output_transformation)

    if deformation_file is None:
        command = ['transformix',
                   '-tp', input_transformation,
                   '-out', output_dir,
                   '-def', 'all']

        if verbose:
            print('Running transformix...')

        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()

        if proc.returncode == 0:
            if verbose:
                print('Finished successfully!')
        else:
            if verbose:
                print('Encountered errors:\n{}'.format(stderr))
                print('Aborting!')
            return

        # Create the output deformation field file
        if verbose:
            print('Clean up output...')
        output_deformation = os.path.splitext(output_transformation)[0] + '.def.nii.gz'
        os.rename(os.path.join(output_dir, 'deformationField.nii.gz'), output_deformation)
    else:
        output_deformation = deformation_file

    # Remove log
    if os.path.exists(os.path.join(output_dir, 'transformix.log')):
        os.remove(os.path.join(output_dir, 'transformix.log'))

    # Create the deformation file
    if verbose:
        print('Creating transformation file...')
    create_def_transform_file(input_transformation, output_transformation, os.path.abspath(output_deformation))


def create_def_transform_file(input_transformation, output_transformation, deformation_field):
    """
    This function reads an elastix transform file and creates a deformation transformation.
    It create the fields for the deformation tranformation and copies the fields related to
    the output sampling etc from the input transformation.

    :param str input_transformations: the path of the input transformation file
    :param str output_transformation: path of the output file to be created
    """
    if not isinstance(input_transformation, str):
        raise TypeError('Input transformation need to be a str containing the path of the transformation file!')

    if not isinstance(output_transformation, str):
        raise TypeError('Output transformation needs to be a str where to write the new transformation to')

    # Read the input transform
    input_transform = ElastixParameterFile(input_transformation)

    # Create a ElastixParameterFile object
    output_transform = ElastixParameterFile()
    output_transform.addline(comment=' Initial transform')
    output_transform['InitialTransformParametersFileName'] = 'NoInitialTransform'
    output_transform['HowToCombineTransforms'] = 'Compose'
    output_transform.addline()
    output_transform.addline(comment=' Deformation transform')
    output_transform['Transform'] = 'DeformationFieldTransform'
    output_transform['DeformationFieldFileName'] = deformation_field
    output_transform['DeformationFieldInterpolationOrder'] = 1
    output_transform['NumberOfParameters'] = 0

    # Copy fields from input transform to output
    output_transform.addline()
    output_transform.addline(comment=' Image specific')
    output_transform['FixedImageDimension'] = input_transform['FixedImageDimension']
    output_transform['MovingImageDimension'] = input_transform['MovingImageDimension']
    output_transform['FixedInternalImagePixelType'] = input_transform['FixedInternalImagePixelType']
    output_transform['MovingInternalImagePixelType'] = input_transform['MovingInternalImagePixelType']
    output_transform.addline()
    output_transform.addline(comment=' ResampleInterpolator specific')
    output_transform['ResampleInterpolator'] = input_transform['ResampleInterpolator']
    output_transform['FinalBSplineInterpolationOrder'] = input_transform['FinalBSplineInterpolationOrder']
    output_transform.addline()
    output_transform.addline(comment=' Resampler specific')
    output_transform['Resampler'] = input_transform['Resampler']
    output_transform['DefaultPixelValue'] = input_transform['DefaultPixelValue']
    output_transform['ResultImageFormat'] = input_transform['ResultImageFormat']
    output_transform['ResultImagePixelType'] = input_transform['ResultImagePixelType']

    # Save transform to file
    output_transform.write(output_transformation)


if __name__ == '__main__':
    # Parse arguments and call main function
    parser = argparse.ArgumentParser(description='Create a deformation field transform based on another elastix transformation file')
    parser.add_argument('--in', metavar='IN', dest='input', type=str, required=True, help='The transform convert to deformation field tranform')
    parser.add_argument('--out', metavar='OUT', dest='output', type=str, required=True, help='The path to write the resulting transform to')
    parser.add_argument('--def', metavar='DEFORMATION_FIELD', dest='deform', type=str, required=False,
                        help='DeformationField to use for the Transform (if not given, it creates one using the elastix on the PATH')

    args = parser.parse_args()
    create_def_transform(args.input, args.output, args.deform, verbose=True)



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PREDICT.plotting.plot_ranked_scores import plot_ranked_scores
import argparse


def main():
    parser = argparse.ArgumentParser(description='Plot the ranked scores of an estimator')
    parser.add_argument('-estimator', '--estimator', metavar='estimator',
                        nargs='+', dest='estimator', type=str, required=True,
                        help='Estimator file (HDF)')
    parser.add_argument('-pinfo', '--pinfo', metavar='pinfo',
                        nargs='+', dest='pinfo', type=str, required=True,
                        help='Patient Info File (txt)')
    parser.add_argument('-ensemble', '--ensemble', metavar='ensemble',
                        nargs='+', dest='ensemble', type=str, required=False,
                        help='Length of ensemble (int)')
    parser.add_argument('-scores', '--scores', metavar='scores',
                        nargs='+', dest='scores', type=str, required=False,
                        help='Type of scoring used: percentages or posteriors (string)')
    parser.add_argument('-label_type', '--label_type', metavar='label_type',
                        nargs='+', dest='label_type', type=str, required=False,
                        help='Label name that is predicted (string)')
    parser.add_argument('-images', '--images', metavar='images',
                        nargs='+', dest='images', type=str, required=False,
                        help='Paths to images (ITK Image files)')
    parser.add_argument('-segmentations', '--segmentations',
                        metavar='segmentations', nargs='+',
                        dest='segmentations', type=str, required=False,
                        help='Paths to segmentations (ITK Image files)')
    parser.add_argument('-output_csv', '--output_csv', metavar='output_csv',
                        nargs='+', dest='output_csv', type=str, required=False,
                        help='File to write output to (csv)')
    parser.add_argument('-output_zip', '--output_zip', metavar='output_zip',
                        nargs='+', dest='output_zip', type=str, required=False,
                        help='File to write output to (zip)')
    args = parser.parse_args()

    # convert inputs that should be single arguments to lists
    pinfo = args.pinfo
    if type(pinfo) is list:
        pinfo = ''.join(pinfo)

    estimator = args.estimator
    if type(estimator) is list:
        estimator = ''.join(estimator)

    ensemble = args.ensemble
    if type(ensemble) is list:
        ensemble = int(ensemble[0])

    label_type = args.label_type
    if type(label_type) is list:
        label_type = ''.join(label_type)

    scores = args.scores
    if type(scores) is list:
        scores = ''.join(scores)

    output_csv = args.output_csv
    if type(output_csv) is list:
        output_csv = ''.join(output_csv)

    output_zip = args.output_zip
    if type(output_zip) is list:
        output_zip = ''.join(output_zip)

    plot_ranked_scores(estimator=estimator,
                       pinfo=pinfo,
                       label_type=label_type,
                       scores=scores,
                       images=args.images,
                       segmentations=args.segmentations,
                       ensemble=ensemble,
                       output_csv=output_csv,
                       output_zip=output_zip)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import SimpleITK as sitk
import fastr
import numpy as np
import os
import WORC.addexceptions as WORCexceptions


class Elastix(object):
    def __init__(self):
        # Which version of elastix and transformix tools should be used
        self.elastix_toolname = 'Elastix'
        self.transformix_toolname = 'Transformix'

        # self.Elastix = sitk.SimpleElastix()
        self.create_network('pairwise')
        self.FixedImage = []
        self.MovingImage = []
        self.FixedMask = []
        self.MovingMask = []
        self.ToTransform = []
        self.ParameterMaps = []  # sitk.VectorOfParameterMap()

        self.TransformedImage = 'vfs://tmp/WORC_Elastix/results/elastix_output_image_{sample_id}_{cardinality}.nii.gz'
        self.TransformedSeg = 'vfs://tmp/WORC_Elastix/results/elastix_output_seg_{sample_id}_{cardinality}.nii.gz'
        self.TransformParameters = 'vfs://tmp/WORC_Elastix/results/elastix_output_trans_{sample_id}_{cardinality}.txt'
        self.fastr_tmpdir = os.path.join(fastr.config.mounts['tmp'], 'WORC_Elastix')
        # TODO: Add initial transformation

    def getparametermap(self, model='affine', size=(512, 512, 128)):
        nvoxels = size[0]*size[1]*size[2]
        if model == 'rigid':
            parameter_map = sitk.GetDefaultParameterMap("rigid")
            parameter_map["AutomaticTransformInitialization"] = ["true"]
            parameter_map["Metric"] = ["AdvancedMattesMutualInformation"]
            parameter_map["NumberOfResolutions"] = ["4"]  # save time/memory
            parameter_map["NumberOfSpatialSamples"] = [str(nvoxels*0.0001)]  # save time/memory
            parameter_map["MaximumNumberOfIterations"] = ["1000"]  # save time/memory
            parameter_map["WriteResultImage"] = ["true"]  # save time/memory
            parameter_map["ErodeMask"] = ["true"]

        elif model == 'affine':
            parameter_map = sitk.GetDefaultParameterMap("affine")
            parameter_map["AutomaticTransformInitialization"] = ["false"]
            parameter_map["Metric"] = ["AdvancedMattesMutualInformation"]
            parameter_map["NumberOfResolutions"] = ["4"]  # save time/memory
            parameter_map["NumberOfSpatialSamples"] = [str(nvoxels*0.0001)]  # save time/memory
            parameter_map["MaximumNumberOfIterations"] = ["1000"]  # save time/memory
            parameter_map["WriteResultImage"] = ["true"]  # save time/memory
            parameter_map["ErodeMask"] = ["true"]

        elif model == 'bspline':
            parameter_map = sitk.GetDefaultParameterMap("bspline")
            parameter_map["AutomaticTransformInitialization"] = ["false"]
            # parameter_map["Metric"] = ["AdvancedMattesMutualInformation"]
            parameter_map["Metric"] = ["AdvancedMattesMutualInformation", "TransformBendingEnergyPenalty"]
            parameter_map["Metric0Weight"] = ["1.0"]
            parameter_map["Metric1Weight"] = ["750.0"] # *100 is about 5 percent
            parameter_map["NumberOfResolutions"] = ["4"]  # save time/memory
            parameter_map["NumberOfSpatialSamples"] = [str(nvoxels*0.0001)]  # save time/memory
            parameter_map["WriteResultImage"] = ["true"]  # save time/memory
            parameter_map["MaximumNumberOfIterations"] = ["1000"]  # save time/memory
            # parameter_map["ImageSampler"] = ["RandomSamplerSparseMask"]
            parameter_map["ErodeMask"] = ["true"]

        else:
            raise KeyError("Model {} cannot be found!").format(model)

        return parameter_map

    def create_network(self, nettype):
        if nettype == 'pairwise':
            # Create the network
            self.network = fastr.Network(id_="elastix_pair")

            # Create Sources
            self.FixedImageSource = self.network.create_source('ITKImageFile', id_='FixedImage')
            self.FixedMaskSource = self.network.create_source('ITKImageFile', id_='FixedMask')
            self.MovingImageSource = self.network.create_source('ITKImageFile', id_='MovingImage')
            self.MovingMaskSource = self.network.create_source('ITKImageFile', id_='MovingMask')
            self.ToTransformSource = self.network.create_source('ITKImageFile', id_='ToTransform')
            self.ParameterMapSource = self.network.create_source('ElastixParameterFile', id_='ParameterMaps', nodegroup='par')
            # Elastix requires the output folder as a sink
            # self.OutputFolderSource = self.network.create_sink('Directory', id_='Out')

            # Create Elastix node and links
            self.elastix_node = self.network.create_node(self.elastix_toolname, id_='elastix')
            self.elastix_node.inputs['fixed_image'] = self.FixedImageSource.output
            self.elastix_node.inputs['fixed_mask'] = self.FixedMaskSource.output
            self.elastix_node.inputs['moving_image'] = self.MovingImageSource.output
            self.elastix_node.inputs['moving_mask'] = self.MovingMaskSource.output
            # self.OutputFolderSource.input = self.elastix_node.outputs['directory']
            self.link_param = self.network.create_link(self.ParameterMapSource.output, self.elastix_node.inputs['parameters'])
            self.link_param.collapse = 'par'

            # Create Sinks
            self.outtrans = self.network.create_sink('ElastixTransformFile', id_='sink_trans')
            self.outimage = self.network.create_sink('ITKImageFile', id_='sink_image')
            self.outseg = self.network.create_sink('ITKImageFile', id_='sink_seg')
            self.outtrans.inputs['input'] = self.elastix_node.outputs['transform']

            # Transform output image
            self.transformix_node = self.network.create_node(self.transformix_toolname, id_='transformix')
            self.transformix_node.inputs['image'] = self.MovingImageSource.output
            self.transformix_node.inputs['transform'] = self.elastix_node.outputs['transform'][-1]
            self.outimage.inputs['input'] = self.transformix_node.outputs['image']

            # First change the FinalBSplineInterpolationOrder to  0 for the segmentation
            self.changeorder_node = self.network.create_node('EditElastixTransformFile', id_='editelpara')
            self.link_trans = self.network.create_link(self.elastix_node.outputs['transform'][-1], self.changeorder_node.inputs['transform'])
            # self.link_trans.converge = 0
            # self.link_trans.collapse = 'FixedImage'
            # self.link_trans.expand = True

            # Co[y metadata from image to segmentation as Elastix uses this
            self.copymetadata_node = self.network.create_node('CopyMetadata', id_='copymetadata')
            self.copymetadata_node.inputs['source'] = self.MovingImageSource.output
            self.copymetadata_node.inputs['destination'] = self.ToTransformSource.output

            # Then transform the segmentation
            self.transformix_node_seg = self.network.create_node(self.transformix_toolname, id_='transformix_seg')
            self.transformix_node_seg.inputs['image'] = self.copymetadata_node.outputs['output']
            self.transformix_node_seg.inputs['transform'] = self.changeorder_node.outputs['transform'][-1]
            self.outseg.inputs['input'] = self.transformix_node_seg.outputs['image']
        else:
            # Create the network
            self.network = fastr.Network(id_="elastix_group")

            # Create Sources
            self.FixedImageSource = self.network.create_source('ITKImageFile', id_='FixedImage')
            self.FixedMaskSource = self.network.create_source('ITKImageFile', id_='FixedMask')
            self.ToTransformSource = self.network.create_source('ITKImageFile', id_='ToTransform')
            self.ParameterMapSource = self.network.create_source('ElastixParameterFile', id_='ParameterMaps', nodegroup='par')
            # Elastix requires the output folder as a sink
            # self.OutputFolderSource = self.network.create_sink('Directory', id_='Out')

            # Create Elastix node and links
            self.elastix_node = self.network.create_node(self.elastix_toolname, id_='elastix')
            self.elastix_node.inputs['fixed_image'] = self.FixedImageSource.output
            self.elastix_node.inputs['fixed_mask'] = self.FixedMaskSource.output
            self.elastix_node.inputs['moving_image'] = self.FixedImageSource.output
            self.elastix_node.inputs['moving_mask'] = self.FixedMaskSource.output
            # self.OutputFolderSource.input = self.elastix_node.outputs['directory']
            self.link_param = self.network.create_link(self.ParameterMapSource.output, self.elastix_node.inputs['parameters'])
            self.link_param.collapse = 'par'

            # Create Sinks
            self.outtrans = self.network.create_sink('ElastixTransformFile', id_='sink_trans')
            self.outimage = self.network.create_sink('ITKImageFile', id_='sink_image')
            self.outseg = self.network.create_sink('ITKImageFile', id_='sink_seg')
            self.outtrans.inputs['input'] = self.elastix_node.outputs['transform']

            # Transform output image
            self.transformix_node = self.network.create_node(self.transformix_toolname, id_='transformix')
            self.transformix_node.inputs['image'] = self.MovingImageSource.output
            self.transformix_node.inputs['transform'] = self.elastix_node.outputs['transform'][-1]
            self.outimage.inputs['input'] = self.transformix_node.outputs['image']

            # First change the FinalBSplineInterpolationOrder to  0 for the segmentation
            self.changeorder_node = self.network.create_node('EditElastixTransformFile', id_='editelpara')
            self.changeorder_node.inputs['set'] = ["FinalBSplineInterpolationOrder=0"]
            self.link_trans = self.network.create_link(self.elastix_node.outputs['transform'], self.changeorder_node.inputs['transform'][-1])
            # self.link_trans.converge = 0
            # self.link_trans.collapse = 'FixedImage'
            # self.link_trans.expand = True

            # Co[y metadata from image to segmentation as Elastix uses this
            self.copymetadata_node = self.network.create_node('CopyMetadata', id_='copymetadata')
            self.copymetadata_node.inputs['source'] = self.MovingImageSource.output
            self.copymetadata_node.inputs['destination'] = self.ToTransformSource.output

            # Then transform the segmentation
            self.transformix_node_seg = self.network.create_node(self.transformix_toolname, id_='transformix_seg')
            self.transformix_node_seg.inputs['image'] = self.copymetadata_node.outputs['output']
            self.transformix_node_seg.inputs['transform'] = self.changeorder_node.outputs['transform'][-1]
            self.outseg.inputs['input'] = self.transformix_node_seg.outputs['image']

    def addchangeorder(self):
        # For the last file, change also the dependence on the previous files
        N_parameterfiles = 1
        sources = list()
        for num in range(0, N_parameterfiles):
            if num != 0:
                # We also need to refer to the correct initial transform files
                intrans = 'InitialTransformParametersFileName=' +\
                    os.path.join(self.fastr_tmpdir, 'editelpara', 'transform_' + str(num - 1) + '.txt')
                sources.append("FinalBSplineInterpolationOrder=0" + intrans)
            else:
                sources.append("FinalBSplineInterpolationOrder=0")

        # self.set = self.network.create_source("AnyType", id_='setorder')
        # self.source_data['setorder'] = sources
        # self.changeorder_node.inputs['set'] = self.set.output
        self.changeorder_node.inputs['set'] = sources

    def create_bbox(self, seg, pad=[2, 25, 25]):
        '''
        Create a bounding box around an input segmentation
        with a certain padding
        '''
        segim = sitk.ReadImage(seg)
        segim = sitk.GetArrayFromImage(segim)
        segim[segim != 0] = 1

        nonzero_x = np.nonzero(np.sum(segim, (1, 2)))[0]
        nonzero_y = np.nonzero(np.sum(segim, (0, 2)))[0]
        nonzero_z = np.nonzero(np.sum(segim, (0, 1)))[0]

        x1, x2 = nonzero_x[0], nonzero_x[-1]
        y1, y2 = nonzero_y[0], nonzero_y[-1]
        z1, z2 = nonzero_z[0], nonzero_z[-1]
        mask = np.zeros(segim.shape)
        x1 = max(0, x1 - pad[0])
        x2 = min(segim.shape[0], x2 + pad[0])
        y1 = max(0, y1 - pad[1])
        y2 = min(segim.shape[1], y2 + pad[1])
        z1 = max(0, z1 - pad[2])
        z2 = min(segim.shape[2], z2 + pad[2])
        mask[x1:x2, y1:y2, z1:z2] = 1
        mask = sitk.GetImageFromArray(mask)

        return mask

    def execute(self):
        # Check if minimal input is supplied
        if not self.FixedImage:
            message = "You need to supply a fixed image for registration."
            raise WORCexceptions.WORCNotImplementedError(message)

        if not self.MovingImage:
            message = "You need to supply a moving image for registration."
            raise WORCexceptions.WORCNotImplementedError(message)

        if len(self.ParameterMaps) == 0:
            message = "You need to supply at leat one parameter map for registration."
            raise WORCexceptions.WORCNotImplementedError(message)

        # Set moving and fixed image sources
        self.source_data = dict()
        self.source_data['FixedImage'] = self.FixedImage
        self.source_data['MovingImage'] = self.MovingImage

        # Create a temporary directory to use
        tempdir = os.path.join(fastr.config.mounts['tmp'], 'WORC_Elastix')
        if not os.path.exists(tempdir):
            os.makedirs(tempdir)

        # Set the parameter map sources
        if type(self.ParameterMaps) is list:
            # Files, thus just provide them to the elastix node
            self.source_data['ParameterMaps'] = self.ParameterMaps
        else:
            # Use SimpleTransformix to save the maps and add them
            SimpleElastix = sitk.SimpleElastix()
            self.source_data['ParameterMaps'] = list()
            for num, f in enumerate(self.ParameterMaps):
                filename = ('ParameterMap{}.txt').format(str(num))
                fname = os.path.join(tempdir, filename)
                SimpleElastix.WriteParameterFile(f, fname)
                sourcename = 'vfs://tmp/WORC_Elastix/' + filename
                self.source_data['ParameterMaps'].append(sourcename)

        # Based on number of parameterfiles, add nodes to train FinalBSplineInterpolationOrder
        self.addchangeorder()

        # Set the mask sources if provided
        # if self.FixedMask is not None:
        self.source_data['FixedMask'] = self.FixedMask

        # if self.MovingMask is not None:
        self.source_data['MovingMask'] = self.MovingMask

        # Add other images to transform if given
        # if self.ToTransform is not None:
        self.source_data['ToTransform'] = self.ToTransform

        # Set the network sinks
        self.sink_data = dict()
        self.sink_data['sink_trans'] = self.TransformParameters
        self.sink_data['sink_image'] = self.TransformedImage
        self.sink_data['sink_seg'] = self.TransformedSeg

        # Set outputfolder if given
        # if self.OutputFolder:
        #     self.sink_data['Out'] = self.OutputFolder
        # else:
        #     self.sink_data['Out'] = 'vfs://tmp/WORC_Elastix/output'

        # print self.sink_data['Out']

        # Execute the network
        self.network.draw_network('WORC_Elastix', img_format='svg', draw_dimension=True)
        self.network.dumpf('{}.json'.format(self.network.id), indent=2)
        self.network.execute(self.source_data, self.sink_data, tmpdir=self.fastr_tmpdir)

        # Automatically read in the images (and parameter maps if requested?)



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import SimpleITK as sitk
import argparse
import dicom as pydicom
import WORC.IOparser.config_preprocessing as config_io
import os


def main():
    parser = argparse.ArgumentParser(description='Feature extraction')
    parser.add_argument('-im', '--im', metavar='image', nargs='+',
                        dest='im', type=str, required=False,
                        help='Images to calculate features on')
    parser.add_argument('-md', '--md', metavar='metadata', dest='md',
                        type=str, required=False, nargs='+',
                        help='Clinical data on patient (DICOM)')
    parser.add_argument('-mask', '--mask', metavar='mask', dest='mask',
                        type=str, required=False, nargs='+',
                        help='Mask that can be used in normalization')
    parser.add_argument('-para', '--para', metavar='Parameters', nargs='+',
                        dest='para', type=str, required=True,
                        help='Parameters')
    parser.add_argument('-out', '--out', metavar='Features',
                        dest='out', type=str, required=False,
                        help='Image output (ITK Image)')
    args = parser.parse_args()

    # Convert list inputs to strings
    if type(args.im) is list:
        args.im = ''.join(args.im)

    if type(args.md) is list:
        args.md = ''.join(args.md)

    if type(args.mask) is list:
        args.mask = ''.join(args.mask)

    if type(args.para) is list:
        args.para = ''.join(args.para)

    if type(args.out) is list:
        args.out = ''.join(args.out)

    # Read the config, image and if given masks and metadata
    config = config_io.load_config(args.para)
    image = sitk.ReadImage(args.im)

    if args.md is not None:
        metadata = pydicom.read_file(args.md)

    if args.mask is not None:
        mask = sitk.ReadImage(args.mask)

    # Convert image to Hounsfield units if type is CT
    image_type = config['ImageFeatures']['image_type']
    # NOTE: We only do this if the input is a DICOM folder
    if 'CT' in image_type and not os.path.isfile(args.im):
        print('Converting intensity to Hounsfield units.')
        image = image*metadata.RescaleSlope +\
            metadata.RescaleIntercept

    # Apply the preprocessing
    if not config['Normalize']['ROI']:
        print('Apply z-scoring on full image.')
        image = sitk.Normalize(image)
    else:
        print('Apply scaling of image based on a Region Of Interest.')
        if args.mask is None:
            raise IOError('Mask input required for ROI normalization.')
        else:
            if config['Normalize']['Method'] == 'z_score':
                # Apply scaling using z-scoring based on the ROI

                # Cast to float to allow proper processing
                image = sitk.Cast(image, 9)

                LabelFilter = sitk.LabelStatisticsImageFilter()
                LabelFilter.Execute(image, mask)
                ROI_mean = LabelFilter.GetMean(1)
                ROI_std = LabelFilter.GetSigma(1)

                image = sitk.ShiftScale(image,
                                        shift=-ROI_mean,
                                        scale=1.0/ROI_std)
            elif config['Normalize']['Method'] == 'minmed':
                # Apply scaling using the minimum and mean of the ROI
                image = sitk.Cast(image, 9)

                LabelFilter = sitk.LabelStatisticsImageFilter()
                LabelFilter.Execute(image, mask)
                ROI_median = LabelFilter.GetMedian(1)
                ROI_minimum = LabelFilter.GetMinimum(1)

                image = sitk.ShiftScale(image,
                                        shift=-ROI_minimum,
                                        scale=0.5/ROI_median)

    # Save the output
    sitk.WriteImage(image, args.out)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2011-2017 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import configparser
import fastr
import os
from random import randint
import addexceptions as WORCexceptions
import IOparser.config_WORC as config_io
from tools.Elastix import Elastix


class WORC(object):
    """A Workflow for Optimal Radiomics Classification object with the
       following properties:

    Attributes:
        name: name of the network.
        configs: Configuration parameters (list, required)
        labels: labels of patients (required)
        network: the FASTR network to use. Needs to be generated through "build".
        images: to be used for Radiomics computation (list, optional)
        segmentations: annotations for images (list, optional)
        semantics: semantic features per image type (list, optional)
        masks: state which pixels of images are valid (list, optional)
        features: input Radiomics features for classification (list, optional)
        metadata: DICOM headers belonging to images (list, optional)
        Elastix_Para: parameter files for Elastix (list, optional)
        fastr_plugin: plugin to use for FASTR execution
        fastr_tempdir: temporary directory to use for FASTR execution
        additions: additional inputs for your network (dict, optional)
        source_data: data to use as sources for FASTR (dict)
        sink_data: data to use as sinks for FASTR (dict)
    """

    def __init__(self, name=''):
        """Initialize WORC object. Set the initial variables all to None,
           except for some defaults.

        Arguments:
            name: name of the nework (string, optional)

        """
        self.name = name
        self.configs = list()
        self.fastrconfigs = list()
        self.images_train = list()
        self.segmentations_train = list()
        self.semantics_train = list()
        self.labels_train = list()
        self.masks_train = list()
        self.features_train = list()
        self.metadata_train = list()

        self.images_test = list()
        self.segmentations_test = list()
        self.semantics_test = list()
        self.labels_test = list()
        self.masks_test = list()
        self.features_test = list()
        self.metadata_test = list()

        self.Elastix_Para = list()
        self.fastr_plugin = 'ProcessPoolExecution'
        if name == '':
            name = [randint(0, 9) for p in range(0,5)]
            self.fastr_tmpdir = os.path.join(fastr.config.mounts['tmp'], 'fastr' + str(name))
        else:
            self.fastr_tmpdir = os.path.join(fastr.config.mounts['tmp'], name)
        self.additions = dict()
        self.Tools = Tools()

    def defaultconfig(self):
        """Generate a configparser object holding all default configuration values.

        Returns:
            config: configparser configuration file

        """
        # TODO: cluster parallel execution parameters
        config = configparser.ConfigParser()
        config.optionxform = str

        # General configuration of WORC
        config['General'] = dict()
        config['General']['cross_validation'] = 'True'
        config['General']['Segmentix'] = 'False'
        config['General']['PCE'] = 'False'  # We do not yet provide this module
        config['General']['FeatureCalculator'] = 'CalcFeatures'
        config['General']['RegistrationNode'] = "Elastix"
        config['General']['TransformationNode'] = "Transformix"

        # Segmentix
        config['Segmentix'] = dict()
        config['Segmentix']['mask'] = 'subtract'
        config['Segmentix']['segtype'] = 'None'
        config['Segmentix']['segradius'] = '5'
        config['Segmentix']['fillholes'] = 'True'
        config['Segmentix']['N_blobs'] = '1'

        # Gridsearch options
        config['SelectFeatGroup'] = dict()
        config['SelectFeatGroup']['shape_features'] = 'True'
        config['SelectFeatGroup']['histogram_features'] = 'True'
        config['SelectFeatGroup']['orientation_features'] = 'True'
        config['SelectFeatGroup']['texture_features'] = 'True'
        config['SelectFeatGroup']['patient_features'] = 'False'
        config['SelectFeatGroup']['semantic_features'] = 'False'
        config['SelectFeatGroup']['coliage_features'] = 'False'

        # PREDICT - Feature calculation
        config['ImageFeatures'] = dict()
        config['ImageFeatures']['orientation'] = 'True'
        config['ImageFeatures']['texture'] = 'all'
        config['ImageFeatures']['coliage'] = 'False'

        # Defines what should be done with the images
        config['ImageFeatures']['image_type'] = 'CT'

        # Define frequencies for gabor filter in pixels
        config['ImageFeatures']['gabor_frequencies'] = '0.05, 0.2, 0.5'

        # Gabor angles in degrees
        config['ImageFeatures']['gabor_angles'] = '0, 45, 90, 135'

        # PREDICT - Feature selection
        config['Featsel'] = dict()
        config['Featsel']['Variance'] = 'True'

        # PREDICT - Classification
        config['Classification'] = dict()
        config['Classification']['fastr'] = 'True'
        config['Classification']['classifier'] = 'SVM'
        config['Classification']['Kernel'] = 'polynomial'

        config['CrossValidation'] = dict()
        config['CrossValidation']['N_iterations'] = '50'
        config['CrossValidation']['test_size'] = '0.2'

        config['Genetics'] = dict()
        config['Genetics']['mutation_type'] = '[GP]'
        config['Genetics']['url'] = 'WIP'
        config['Genetics']['projectID'] = 'WIP'

        config['HyperOptimization'] = dict()
        config['HyperOptimization']['scoring_method'] = 'f1_weighted'
        config['HyperOptimization']['test_size'] = '0.15'
        config['HyperOptimization']['N_iterations'] = '10000'
        config['HyperOptimization']['n_jobspercore'] = '2000'  # only relevant when using fastr in classification

        config['FeatureScaling'] = dict()
        config['FeatureScaling']['scale_features'] = 'True'
        config['FeatureScaling']['scaling_method'] = 'z_score'

        config['SampleProcessing'] = dict()
        config['SampleProcessing']['SMOTE'] = 'True'
        config['SampleProcessing']['SMOTE_ratio'] = '1.0'

        # BUG: the FASTR XNAT plugin can only retreive folders. We therefore need to add the filenames of the resources manually
        config['FASTR_bugs'] = dict()
        config['FASTR_bugs']['images'] = 'image.nii.gz'
        config['FASTR_bugs']['segmentations'] = 'mask.nii.gz'

        return config

    def build(self, wtype='training'):
        """Build the training network based on the given attributes."""

        if wtype == 'training':
            self.build_training()
            self.wtype = wtype
        elif wtype == 'testing':
            self.build_testing()
            self.wtype = wtype

    def build_training(self):
        """Build the training network based on the given attributes."""

        # FIXME: Currently, we do the same routine for the test and training sets

        # We either need images or features for Radiomics
        if self.images_train or self.features_train:
            # We currently require labels for supervised learning
            if self.labels_train:
                if not self.configs:
                    print("No configuration given, assuming default")
                    if self.images_train:
                        self.configs = [self.defaultconfig()] * len(self.images_train)
                    else:
                        self.configs = [self.defaultconfig()] * len(self.features_train)
                self.network = fastr.Network('WORC' + self.name)

                # BUG: We currently use the first configuration as general config
                image_types = list()
                for c in range(len(self.configs)):
                    if type(self.configs[c]) == str():
                        # Probably, c is a configuration file
                        self.configs[c] = config_io.load_config(c)

                    image_types.append(self.configs[c]['ImageFeatures']['image_type'])

                # Classification tool and label source
                self.network.source_patientclass_train = self.network.create_source('PatientInfoFile', id_='patientclass', nodegroup='pctrain')
                if self.labels_test:
                    self.network.source_patientclass_test = self.network.create_source('PatientInfoFile', id_='patientclass', nodegroup='pctest')

                self.network.classify = self.network.create_node('TrainClassifier_new', memory='12G', id_='classify')

                # Outputs
                self.network.sink_classification = self.network.create_sink('HDF5', id_='classification')
                self.network.sink_performance = self.network.create_sink('JsonFile', id_='performance')

                # Links
                self.network.source_class_config = self.network.create_source('ParameterFile', id_='config_classification', nodegroup='conf')
                # self.network.source_class_parameters = self.network.create_source('JsonFile', id_='parameters_classification')
                # self.network.classify.inputs['parameters'] = self.network.source_class_parameters.output
                self.network.link_class_1 = self.network.create_link(self.network.source_class_config.output, self.network.classify.inputs['config'])
                self.network.link_class_2 = self.network.create_link(self.network.source_patientclass_train.output, self.network.classify.inputs['patientclass_train'])
                self.network.link_class_1.collapse = 'conf'
                self.network.link_class_2.collapse = 'pctrain'

                if self.images_test or self.features_test:
                    # FIXME: the naming here is ugly
                    self.network.link_class_3 = self.network.create_link(self.network.source_patientclass_test.output, self.network.classify.inputs['patientclass_test'])
                    self.network.link_class_3.collapse = 'pctest'

                self.network.sink_classification.input = self.network.classify.outputs['classification']
                self.network.sink_performance.input = self.network.classify.outputs['performance']

                if not self.features_train:
                    # Create nodes to compute features
                    self.network.sources_parameters = dict()

                    self.network.calcfeatures_train = dict()
                    self.network.sources_images_train = dict()
                    self.network.sinks_features_train = dict()
                    self.network.converters_im_train = dict()
                    self.network.converters_seg_train = dict()
                    self.network.links_C1_train = dict()

                    if self.images_test or self.features_test:
                        self.network.calcfeatures_test = dict()
                        self.network.sources_images_test = dict()
                        self.network.sinks_features_test = dict()
                        self.network.converters_im_test = dict()
                        self.network.converters_seg_test = dict()
                        self.network.links_C1_test = dict()

                    # Check which nodes are necessary
                    if not self.segmentations_train:
                        message = "No automatic segmentations implemented."
                        raise WORCexceptions.WORCNotImplementedError(message)

                    elif len(self.segmentations_train) == len(image_types):
                        # Segmentations provided
                        self.network.sources_segmentation_train = dict()
                        self.network.sources_segmentation_test = dict()
                        segmode = 'Provided'

                    elif len(self.segmentations_train) == 1:
                        # Assume segmentations need to be registered to other modalities
                        self.network.sources_segmentation = dict()
                        segmode = 'Register'

                        self.network.source_Elastix_Parameters = dict()
                        self.network.elastix_nodes_train = dict()
                        self.network.transformix_nodes_train = dict()
                        self.network.sources_segmentations_train = dict()
                        self.network.sinks_transformations_train = dict()
                        self.network.sinks_segmentations_train = dict()
                        self.network.converters_seg_train = dict()

                        self.network.elastix_nodes_test = dict()
                        self.network.transformix_nodes_test = dict()
                        self.network.sources_segmentations_test = dict()
                        self.network.sinks_transformations_test = dict()
                        self.network.sinks_segmentations_test = dict()
                        self.network.converters_seg_test = dict()
                        pass

                    # BUG: We assume that first type defines if we use segmentix
                    if self.configs[0]['General']['Segmentix'] == 'True':
                        # Use the segmentix toolbox for segmentation processing
                        self.network.sinks_segmentation_train = dict()
                        self.network.sources_masks_train = dict()
                        self.network.converters_masks_train = dict()
                        self.network.nodes_segmentix_train = dict()

                        if self.images_test or self.features_test:
                            self.network.sinks_segmentation_test = dict()
                            self.network.sources_masks_test = dict()
                            self.network.converters_masks_test = dict()
                            self.network.nodes_segmentix_test = dict()

                    if self.semantics_train:
                        self.network.sources_semantics_train = dict()

                    if self.metadata_train:
                        self.network.sources_metadata_train = dict()

                    if self.semantics_test:
                        self.network.sources_semantics_test = dict()

                    if self.metadata_test:
                        self.network.sources_metadata_test = dict()

                    # Create label for each modality/image
                    self.modlabels = list()
                    for num, mod in enumerate(image_types):
                        num = 0
                        label = mod + '_' + str(num)
                        while label in self.network.calcfeatures_train.keys():
                            # if label already exists, add number to label
                            num += 1
                            label = mod + str(num)
                        self.modlabels.append(label)

                        # Create required sources and sinks
                        self.network.sources_parameters[label] = self.network.create_source('ParameterFile', id_='parameters_' + label)
                        self.network.sources_images_train[label] = self.network.create_source('ITKImageFile', id_='images_train_' + label, nodegroup='train')
                        self.network.sinks_features_train[label] = self.network.create_sink('HDF5', id_='features_train_' + label)
                        if self.images_test or self.features_test:
                            self.network.sources_images_test[label] = self.network.create_source('ITKImageFile', id_='images_test_' + label, nodegroup='test')
                            self.network.sinks_features_test[label] = self.network.create_sink('HDF5', id_='features_test_' + label)

                        if self.metadata_train:
                            self.network.sources_metadata_train[label] = self.network.create_source('DicomImageFile', id_='metadata_train_' + label, nodegroup='train')

                        if self.metadata_test:
                            self.network.sources_metadata_test[label] = self.network.create_source('DicomImageFile', id_='metadata_test_' + label, nodegroup='test')

                        if self.masks_train:
                            # Create mask source and convert
                            self.network.sources_masks_train[label] = self.network.create_source('ITKImageFile', id_='mask_train_' + label, nodegroup='train')
                            self.network.converters_masks_train[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_mask_train_' + label, nodegroup='train')
                            self.network.converters_masks_train[label].inputs['image'] = self.network.sources_masks_train[label].output

                        if self.masks_test:
                            # Create mask source and convert
                            self.network.sources_masks_test[label] = self.network.create_source('ITKImageFile', id_='mask_test_' + label, nodegroup='test')
                            self.network.converters_masks_test[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_mask_test_' + label, nodegroup='test')
                            self.network.converters_masks_test[label].inputs['image'] = self.network.sources_masks_test[label].output

                        # First convert the images
                        if any(modality in mod for modality in ['MR', 'CT', 'MG', 'PET']):
                            # Use ITKTools PXCastConvet for converting image formats
                            self.network.converters_im_train[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_im_train_' + label)
                            if self.images_test or self.features_test:
                                self.network.converters_im_test[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_im_test_' + label)

                        elif 'DTI' in mod:
                            # TODO: This reader is currently missing
                            self.network.converters_im_train[label] = self.network.create_node('DTIreader', memory='4G', id_='convert_im_train_' + label)
                            if self.images_test or self.features_test:
                                self.network.converters_im_test[label] = self.network.create_node('DTIreader', memory='4G', id_='convert_im_test_' + label)

                        else:
                            raise WORCexceptions.WORCTypeError(('No valid image type for modality {}: {} provided.').format(str(num), mod))

                        calcfeat_node = str(self.configs[num]['General']['FeatureCalculator'])
                        self.network.calcfeatures_train[label] = self.network.create_node(calcfeat_node, memory='14G', id_='calcfeatures_train_' + label)
                        if self.images_test or self.features_test:
                            self.network.calcfeatures_test[label] = self.network.create_node(calcfeat_node, memory='14G', id_='calcfeatures_test_' + label)

                        # Create required links
                        self.network.converters_im_train[label].inputs['image'] = self.network.sources_images_train[label].output
                        self.network.calcfeatures_train[label].inputs['parameters'] = self.network.sources_parameters[label].output
                        self.network.calcfeatures_train[label].inputs['image'] = self.network.converters_im_train[label].outputs['image']

                        if self.images_test or self.features_test:
                            self.network.converters_im_test[label].inputs['image'] = self.network.sources_images_test[label].output
                            self.network.calcfeatures_test[label].inputs['parameters'] = self.network.sources_parameters[label].output
                            self.network.calcfeatures_test[label].inputs['image'] = self.network.converters_im_test[label].outputs['image']

                        if self.metadata_train:
                            self.network.calcfeatures_train[label].inputs['metadata'] = self.network.sources_metadata_train[label].output

                        if self.metadata_train:
                            self.network.calcfeatures_train[label].inputs['metadata'] = self.network.sources_metadata_train[label].output

                        if self.semantics_train:
                            self.network.sources_semantics_train[label] = self.network.create_source('CSVFile', id_='semantics_train_' + label)
                            self.network.calcfeatures_train[label].inputs['semantics'] = self.network.sources_semantics_train[label].output

                        if self.semantics_test:
                            self.network.sources_semantics_test[label] = self.network.create_source('CSVFile', id_='semantics_test_' + label)
                            self.network.calcfeatures_test[label].inputs['semantics'] = self.network.sources_semantics_test[label].output

                        if segmode == 'Provided':
                            # Use the provided segmantions for each modality
                            self.network.sources_segmentation_train[label] = self.network.create_source('ITKImageFile', id_='segmentation_train_' + label, nodegroup='train')
                            self.network.converters_seg_train[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_seg_train_' + label)
                            self.network.converters_seg_train[label].inputs['image'] = self.network.sources_segmentation_train[label].output

                            if self.images_test or self.features_test:
                                self.network.sources_segmentation_test[label] = self.network.create_source('ITKImageFile', id_='segmentation_test_' + label, nodegroup='test')
                                self.network.converters_seg_test[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_seg_test_' + label)
                                self.network.converters_seg_test[label].inputs['image'] = self.network.sources_segmentation_test[label].output

                        elif segmode == 'Register':
                            # Align segmentation of first modality to others using registration with Elastix

                            # Create sources and converter for segmentation
                            self.network.sources_segmentations_train[label] = self.network.create_source('ITKImageFile', id_='segmentations_train_' + label, nodegroup='input')
                            self.network.converters_seg_train[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_seg_train_' + label)
                            self.network.converters_seg_train[label].inputs['image'] = self.network.sources_segmentations_train[label].output

                            if self.images_test or self.features_test:
                                self.network.sources_segmentations_test[label] = self.network.create_source('ITKImageFile', id_='segmentations_test_' + label, nodegroup='input')
                                self.network.converters_seg_test[label] = self.network.create_node('PxCastConvert', memory='4G', id_='convert_seg_test_' + label)
                                self.network.converters_seg_test[label].inputs['image'] = self.network.sources_segmentations_test[label].output

                            # Assume provided segmentation is on first modality
                            if num > 0:
                                # Use elastix and transformix for registration
                                # NOTE: Assume elastix node type is on first configuration
                                elastix_node = str(self.configs[0]['General']['RegistrationNode'])
                                transformix_node = str(self.configs[0]['General']['TransformationNode'])
                                self.network.elastix_nodes_train[label] = self.network.create_node(elastix_node, id_='elastix_train' + label)
                                self.network.transformix_nodes_train[label] = self.network.create_node(transformix_node, id_='transformix_train_' + label)

                                if self.images_test or self.features_test:
                                    self.network.elastix_nodes_test[label] = self.network.create_node(elastix_node, id_='elastix_test_' + label)
                                    self.network.transformix_nodes_test[label] = self.network.create_node(transformix_node, id_='transformix_test_' + label)

                                # create sources_segmentation
                                # M1 = moving, others = fixed
                                self.network.elastix_nodes_train[label].inputs['fixed_image'] = self.network.converters_im_train[label].outputs['image']
                                self.network.elastix_nodes_train[label].inputs['moving_image'] = self.network.converters_im_train[self.modlabels[0]].outputs['image']
                                self.network.transformix_nodes_train[label].inputs['image'] = self.network.converters_seg_train[self.modlabels[0]].outputs['image']

                                if self.images_test or self.features_test:
                                    self.network.elastix_nodes_test[label].inputs['fixed_image'] = self.network.converters_im_test[label].outputs['image']
                                    self.network.elastix_nodes_test[label].inputs['moving_image'] = self.network.converters_im_test[self.modlabels[0]].outputs['image']
                                    self.network.transformix_nodes_test[label].inputs['image'] = self.network.converters_seg_test[self.modlabels[0]].outputs['image']

                                # Apply registration to input modalities
                                self.network.source_Elastix_Parameters[label] = self.network.create_source('ElastixParameterFile', id_='Elastix_Para_' + label, nodegroup='elpara')
                                self.link_elparam_train = self.network.create_link(self.network.source_Elastix_Parameters[label].output,
                                                                           self.network.elastix_nodes_train[label].inputs['parameters'])
                                self.link_elparam_train.collapse = 'elpara'

                                if self.images_test or self.features_test:
                                    self.link_elparam_test = self.network.create_link(self.network.source_Elastix_Parameters[label].output,
                                                                               self.network.elastix_nodes_test[label].inputs['parameters'])
                                    self.link_elparam_test.collapse = 'elpara'

                                if self.masks_train:
                                    self.network.elastix_nodes_train[label].inputs['fixed_mask'] = self.network.converters_masks_train[label].outputs['image']
                                    self.network.elastix_nodes_train[label].inputs['moving_mask'] = self.network.converters_masks_train[self.modlabels[0]].outputs['image']

                                if self.images_test or self.features_test:
                                    if self.masks_test:
                                        self.network.elastix_nodes_test[label].inputs['fixed_mask'] = self.network.converters_masks_test[label].outputs['image']
                                        self.network.elastix_nodes_test[label].inputs['moving_mask'] = self.network.converters_masks_test[self.modlabels[0]].outputs['image']

                                # Link data and transformation to transformix and source
                                self.network.transformix_nodes_train[label].inputs['transform'] = self.network.elastix_nodes_train[label].outputs['transform']
                                self.network.calcfeatures_train[label].inputs['segmentation'] = self.network.transformix_nodes_train[label] .outputs['image']

                                if self.images_test or self.features_test:
                                    self.network.transformix_nodes_test[label].inputs['transform'] = self.network.elastix_nodes_test[label].outputs['transform']
                                    self.network.calcfeatures_test[label].inputs['segmentation'] = self.network.transformix_nodes_test[label] .outputs['image']

                                # Save output
                                self.network.sinks_transformations_train[label] = self.network.create_sink('ElastixTransformFile', id_='transformation_train_' + label)
                                self.network.sinks_segmentations_train[label] = self.network.create_sink('ITKImageFile', id_='segmentations_out_train_' + label)
                                self.network.sinks_transformations_train[label].input = self.network.elastix_nodes_train[label].outputs['transform']
                                self.network.sinks_segmentations_train[label].input = self.network.transformix_nodes_train[label].outputs['image']

                                if self.images_test or self.features_test:
                                    self.network.sinks_transformations_test[label] = self.network.create_sink('ElastixTransformFile', id_='transformation_test_' + label)
                                    self.network.sinks_segmentations_test[label] = self.network.create_sink('ITKImageFile', id_='segmentations_out_test_' + label)
                                    self.network.sinks_transformations_test[label].input = self.network.elastix_nodes_test[label].outputs['transform']
                                    self.network.sinks_segmentations_test[label].input = self.network.transformix_nodes_test[label].outputs['image']

                        if self.configs[num]['General']['Segmentix'] == 'True':
                            # Use segmentix node to convert input segmentation into correct contour
                            self.network.sinks_segmentation_train[label] = self.network.create_sink('ITKImageFile', id_='segmentation_out_train_' + label)
                            self.network.nodes_segmentix_train[label] = self.network.create_node('Segmentix', memory='6G', id_='segmentix_train_' + label)
                            self.network.nodes_segmentix_train[label].inputs['segmentation_in'] = self.network.converters_seg_train[label].outputs['image']
                            self.network.nodes_segmentix_train[label].inputs['parameters'] = self.network.sources_parameters[label].output
                            self.network.calcfeatures_train[label].inputs['segmentation'] = self.network.nodes_segmentix_train[label].outputs['segmentation_out']
                            self.network.sinks_segmentation_train[label].input = self.network.nodes_segmentix_train[label].outputs['segmentation_out']

                            if self.images_test or self.features_test:
                                self.network.sinks_segmentation_test[label] = self.network.create_sink('ITKImageFile', id_='segmentation_out_test_' + label)
                                self.network.nodes_segmentix_test[label] = self.network.create_node('Segmentix', memory='6G', id_='segmentix_test_' + label)
                                self.network.nodes_segmentix_test[label].inputs['segmentation_in'] = self.network.converters_seg_test[label].outputs['image']
                                self.network.nodes_segmentix_test[label].inputs['parameters'] = self.network.sources_parameters[label].output
                                self.network.calcfeatures_test[label].inputs['segmentation'] = self.network.nodes_segmentix_test[label].outputs['segmentation_out']
                                self.network.sinks_segmentation_test[label].input = self.network.nodes_segmentix_test[label].outputs['segmentation_out']

                            if self.masks_train:
                                # Use masks
                                self.network.nodes_segmentix_train[label].inputs['mask'] = self.network.converters_masks_train[label].outputs['image']

                            if self.masks_test:
                                # Use masks
                                self.network.nodes_segmentix_test[label].inputs['mask'] = self.network.converters_masks_test[label].outputs['image']

                        else:
                            self.network.calcfeatures_train[label].inputs['segmentation'] = self.network.converters_seg_train[label].outputs['image']

                            if self.images_test or self.features_test:
                                self.network.calcfeatures_train[label].inputs['segmentation'] = self.network.converters_seg_train[label].outputs['image']

                        # FIXME: we need a feature combiner tool here: now monomodal
                        self.network.links_C1_train[label] = self.network.create_link(self.network.calcfeatures_train[label].outputs['features'], self.network.classify.inputs['features_train'])
                        self.network.links_C1_train[label].collapse = 'train'

                        if self.images_test or self.features_test:
                            self.network.links_C1_test[label] = self.network.create_link(self.network.calcfeatures_test[label].outputs['features'], self.network.classify.inputs['features_test'])
                            self.network.links_C1_test[label].collapse = 'test'

                        # Save output
                        self.network.sinks_features_train[label].input = self.network.calcfeatures_train[label].outputs['features']
                        if self.images_test or self.features_test:
                            self.network.sinks_features_test[label].input = self.network.calcfeatures_test[label].outputs['features']

                else:
                    # Features already provided
                    self.network.sources_features_train = dict()
                    self.network.links_C1_train = dict()

                    if self.features_test:
                        self.network.sources_features_test = dict()
                        self.network.links_C1_test = dict()

                    # Create label for each modality/image
                    self.modlabels = list()
                    for num, mod in enumerate(image_types):
                        num = 0
                        label = mod + str(num)
                        while label in self.network.sources_features_train.keys():
                            # if label exists, add number to label
                            num += 1
                            label = mod + str(num)
                        self.modlabels.append(label)

                    self.network.sources_features_train[label] = self.network.create_source('HDF5', id_='features_train_' + label, nodegroup='train')
                    self.network.links_C1_train[label] = self.network.create_link(self.network.sources_features_train[label].output, self.network.classify.inputs['features_train'])
                    self.network.links_C1_train[label].collapse = 'train'

                    if self.features_test:
                        self.network.sources_features_test[label] = self.network.create_source('HDF5', id_='features_test_' + label, nodegroup='test')
                        self.network.links_C1_test[label] = self.network.create_link(self.network.sources_features_test[label].output, self.network.classify.inputs['features_test'])
                        self.network.links_C1_test[label].collapse = 'test'

                if self.configs[num]['General']['PCE'] == 'True':
                    # NOTE: PCE Features is currently not provided.
                    self.network.PCEnode = self.network.create_node('PCE', memory='64G', id_='PCE')
                    self.network.PCEnode.inputs['svm'] = self.network.classify.outputs['classification']

                    self.network.sink_PCE = self.network.create_sink('MatlabFile', id_='PCE_mat')
                    self.network.sink_SI = self.network.create_sink('MatlabFile', id_='SI_mat')
                    self.network.sink_TS = self.network.create_sink('CSVFile', id_='TS_csv')

                    self.network.sink_PCE.input = self.network.PCEnode.outputs['PCE']
                    self.network.sink_SI.input = self.network.PCEnode.outputs['SI']
                    self.network.sink_TS.input = self.network.PCEnode.outputs['TS']

            else:
                raise WORCexceptions.WORCIOError("Please provide labels.")
        else:
            raise WORCexceptions.WORCIOError("Please provide either images or features.")

    def build_testing(self):
        ''' todo '''

    def set(self):
        """ Set the FASTR source and sink data based on the given attributes."""
        self.fastrconfigs = list()
        self.source_data = dict()
        self.sink_data = dict()

        # If the configuration files are confiparse objects, write to file
        for num, c in enumerate(self.configs):
            if type(c) == configparser.ConfigParser:
                cfile = os.path.join(fastr.config.mounts['tmp'], self.name, ("config_{}_{}.ini").format(self.name, num))
                if not os.path.exists(os.path.dirname(cfile)):
                    os.makedirs(os.path.dirname(cfile))
                with open(cfile, 'w') as configfile:
                    c.write(configfile)
                self.fastrconfigs.append(os.path.join("vfs://tmp/", self.name, ("config_{}_{}.ini").format(self.name, num)))

        # Generate gridsearch parameter files if required
        # NOTE: Below bug is fixed?
        # TODO: We now use the first configuration for the classifier, but his needs to be separated from the rest per modality
        self.source_data['config_classification'] = self.fastrconfigs

        # Set source and sink data
        self.source_data['patientclass'] = self.labels_train

        self.sink_data['classification'] = ("vfs://output/{}/svm_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
        self.sink_data['performance'] = ("vfs://output/{}/performance_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
        if self.configs[0]['General']['PCE']:
            self.sink_data['PCE_mat'] = ("vfs://output/{}/PCE_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            self.sink_data['SI_mat'] = ("vfs://output/{}/SI_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            self.sink_data['TS_csv'] = ("vfs://output/{}/TS_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

        # BUG: this is a bug in the FASTR package. Workaround for nifti XNAT links using expansion of FASTR XNAT plugin.
        for num, im in enumerate(self.images_train):
            if im is not None and 'xnat' in im:
                if 'DICOM' not in im:
                    fastr.ioplugins['xnat']
                    # metalink = im_m1.replace('NIFTI','DICOM')
                    im = fastr.ioplugins['xnat'].expand_url(im)
                    imagename = self.configs[num]['FASTR_bugs']['images']
                    self.images_train[num] = {v[(v.find('subjects') + 9):v.find('/experiments')]: v + imagename + '?insecure=true' for k, v in im}

        for num, seg in enumerate(self.segmentations_train):
            if seg is not None and 'xnat' in seg:
                fastr.ioplugins['xnat']
                seg = fastr.ioplugins['xnat'].expand_url(seg)
                segname = self.configs[num]['FASTR_bugs']['segmentations']
                self.segmentations_train[num] = {v[(v.find('subjects') + 9):v.find('/experiments')]: v + segname + '?insecure=true' for k, v in seg}

        # BUG: if we have only a file on one modality, e.g. semantics, what to do with others? You currently need to provide empty files.
        for num, label in enumerate(self.modlabels):
            self.source_data['parameters_' + label] = self.fastrconfigs[num]

            # Add train data sources
            if self.images_train and len(self.images_train) - 1 >= num:
                self.source_data['images_train_' + label] = self.images_train[num]

            if self.masks_train and len(self.masks_train) - 1 >= num:
                self.source_data['mask_train_' + label] = self.masks_train[num]

            if self.metadata_train and len(self.metadata_train) - 1 >= num:
                self.source_data['metadata_train_' + label] = self.metadata_train[num]

            if self.segmentations_train and len(self.segmentations_train) - 1 >= num:
                self.source_data['segmentations_train_' + label] = self.segmentations_train[num]

            if self.semantics_train and len(self.semantics_train) - 1 >= num:
                self.source_data['semantics_train_' + label] = self.semantics_train[num]

            if self.features_train and len(self.features_train) - 1 >= num:
                self.source_data['features_train_' + label] = self.features_train[num]

            # FIXME: should be for both test and train
            if self.Elastix_Para and len(self.Elastix_Para) - 1 >= num:
                if len(self.Elastix_Para) >= num:
                    self.source_data['Elastix_Para_' + label] = self.Elastix_Para[num]
                else:
                    # Use one fileset for all modalities
                    self.source_data['Elastix_Para_' + label] = self.Elastix_Para[0]

            # Add test data sources
            if self.images_test and len(self.images_test) - 1 >= num:
                self.source_data['images_test_' + label] = self.images_test[num]

            if self.masks_test and len(self.masks_test) - 1 >= num:
                self.source_data['mask_test_' + label] = self.masks_test[num]

            if self.metadata_test and len(self.metadata_test) - 1 >= num:
                self.source_data['metadata_test_' + label] = self.metadata_test[num]

            if self.segmentations_test and len(self.segmentations_test) - 1 >= num:
                self.source_data['segmentations_test_' + label] = self.segmentations_test[num]

            if self.semantics_test and len(self.semantics_test) - 1 >= num:
                self.source_data['semantics_test_' + label] = self.semantics_test[num]

            if self.features_test and len(self.features_test) - 1  >= num:
                self.source_data['features_test_' + label] = self.features_test[num]

            self.sink_data['segmentation_out_train_' + label] = ("vfs://output/{}/seg_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            self.sink_data['features_train_' + label] = ("vfs://output/{}/features_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            self.sink_data['transformation_' + label] = ("vfs://output/{}/transformation_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

            if self.labels_test:
                self.sink_data['segmentation_out_test_' + label] = ("vfs://output/{}/seg_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
                self.sink_data['features_test_' + label] = ("vfs://output/{}/features_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

    def execute(self):
        """ Execute the network through the fastr.network.execute command. """
        # Draw and execute nwtwork
        self.network.draw_network(self.network.id, draw_dimension=True)
        self.network.execute(self.source_data, self.sink_data, execution_plugin=self.fastr_plugin, tmpdir=self.fastr_tmpdir)


class Tools(object):
    def __init__(self):
        self.Elastix = Elastix()



#!/usr/bin/env python
import argparse
import copy
from elastixparameterfile import ElastixParameterFile


def create_mean_transform_bin(input_transformations, output_transformation):
    """
    This function reads a number of elastix transform files, creates the mean transformation and writes it back to the specified output path.

    :param input_transformations: the path of the input transformation files
    :type input_transformations: iterable of str
    :param output_transformation: path of the output file to be created
    """
    if not all(isinstance(x, str) for x in input_transformations):
        raise TypeError('Input transformations need to be a str containing the path of the transformation file!')

    if not isinstance(output_transformation, str):
        raise TypeError('Output transformation needs to be a str where to write the new transformation to')

    input_transformations = [x for x in input_transformations]
    reference_transformation = ElastixParameterFile(input_transformations[0])

    # Outsource to dedicated function
    output_transform = create_mean_transform(reference_transformation, input_transformations)

    # Save transform to file
    output_transform.write(output_transformation)


def create_mean_transform(reference_transform, input_transformations):
    """
    Given a list of transformation paths, and a reference object, create a new object that is the mean transform

    :param ElastixParameterFile reference_transform: The reference transformation object (for defining the target space etc)
    :param input_transformations: The paths of the transforms to create a mean of
    :type input_transformations: list of str
    """
    if not all(isinstance(x, str) for x in input_transformations):
        raise TypeError('Input transformations need to be str paths!')

    if not isinstance(reference_transform, ElastixParameterFile):
        raise TypeError('The reference transform needs to be an ElastixParameterFile object')

    output_transform = copy.deepcopy(reference_transform)

    # Get the weighting vector for the transforms
    weighting = [1.0 / len(input_transformations)] * len(input_transformations)

    # Substitute relevant fields
    output_transform['Transform'] = 'WeightedCombinationTransform'
    output_transform['TransformParameters'] = weighting
    output_transform['NumberOfParameters'] = len(weighting)
    output_transform['InitialTransformParametersFileName'] = 'NoInitialTransform'
    output_transform['ResultImagePixelType'] = 'float'
    output_transform['SubTransforms'] = input_transformations

    return output_transform


if __name__ == '__main__':
    # Parse arguments and call main function
    parser = argparse.ArgumentParser(description='Combine elastix transformation to create a MeanTransform')
    parser.add_argument('--in', metavar='IN', dest='input', type=str, nargs='+', required=True, help='The transforms to combine')
    parser.add_argument('--out', metavar='OUT', dest='output', type=str, required=True, help='The path to write the resulting transform to')

    args = parser.parse_args()
    create_mean_transform_bin(args.input, args.output)



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import SimpleITK as sitk
import radiomics
from radiomics import featureextractor


def AllFeatures(image, mask):
    # Define settings for signature calculation
    # These are currently set equal to the respective default values
    kwargs = {}
    kwargs['binWidth'] = 25
    kwargs['resampledPixelSpacing'] = None  # [3,3,3] is an example for defining resampling (voxels with size 3x3x3mm)
    kwargs['interpolator'] = sitk.sitkBSpline
    kwargs['verbose'] = True

    # Initialize wrapperClass to generate signature
    extractor = featureextractor.RadiomicsFeaturesExtractor(**kwargs)

    # Disable all classes except firstorder
    extractor.enableAllFeatures()

    # Enable writing out the log using radiomics logger
    radiomics.debug()  # Switch on radiomics logging from level=DEBUG (default level=WARNING)

    # Prevent radiomics logger from printing out log entries with level < WARNING to the console
    logger = logging.getLogger('radiomics')
    logger.handlers[0].setLevel(logging.WARNING)
    logger.propagate = False  # Do not pass log messages on to root logger

    # Write out all log entries to a file
    handler = logging.FileHandler(filename='testLog.txt', mode='w')
    formatter = logging.Formatter("%(levelname)s:%(name)s: %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    print("Active features:")
    for cls, features in extractor.enabledFeatures.iteritems():
        if len(features) == 0:
            features = extractor.getFeatureNames(cls)
        for f in features:
            print(f)
            print(eval('extractor.featureClasses["%s"].get%sFeatureValue.__doc__' % (cls, f)))

    print("Calculating features")
    featureVector = extractor.execute(image, mask)

    for featureName in featureVector.keys():
        print("Computed %s: %s" % (featureName, featureVector[featureName]))

    return featureVector



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import WORC.addexceptions as WORCexceptions
import fastr
import os

# NOTE: Very important to give images and segmentations as dict with patient names!


class Evaluate(object):
    def __init__(self, label_type, ensemble=50, scores='percentages',
                 network=None, features=None,
                 fastr_plugin='ProcessPoolExecution',
                 name='Evaluate'):
        '''
        Build a network that evaluates the performance of an estimator.

        Parameters
        ----------

        network: fastr network, default None
                If you input a network, the evaluate network is added
                to the existing network.

        '''
        if network is not None:
            self.network = network
            self.mode = 'WORC'
        else:
            self.network = fastr.Network(id_="Evaluate")
            self.mode = 'StandAlone'
            self.fastr_plugin = fastr_plugin
            self.name = name
            self.fastr_tmpdir = os.path.join(fastr.config.mounts['tmp'], self.name)

        if features is None and self.mode == 'StandAlone':
            raise WORCexceptions.IOError('Either features as input or a WORC network is required for the Evaluate network.')

        self.features = features

        self.label_type = label_type
        self.ensemble = ensemble

        self.create_network()

    def create_network(self):
        '''
        Add evaluate components to network.
        '''

        # Create all nodes
        self.network.node_ROC =\
            self.network.create_node('PlotROC', memory='4G', id_='plot_ROC')
        self.network.node_SVM =\
            self.network.create_node('PlotSVM', memory='4G', id_='plot_SVM')
        self.network.node_Barchart =\
            self.network.create_node('PlotBarchart', memory='4G', id_='plot_Barchart')
        self.network.node_STest =\
            self.network.create_node('StatisticalTestFeatures', memory='4G', id_='statistical_test_features')
        self.network.node_Ranked_Percentages =\
            self.network.create_node('PlotRankedScores', memory='4G', id_='plot_ranked_percentages')
        self.network.node_Ranked_Posteriors =\
            self.network.create_node('PlotRankedScores', memory='4G', id_='plot_ranked_posteriors')

        # Create sinks
        self.network.sink_ROC_PNG =\
            self.network.create_sink('PNGFile', id_='ROC_PNG')
        self.network.sink_ROC_Tex =\
            self.network.create_sink('TexFile', id_='ROC_Tex')
        self.network.sink_ROC_CSV =\
            self.network.create_sink('CSVFile', id_='ROC_CSV')

        self.network.sink_SVM_Json =\
            self.network.create_sink('JsonFile', id_='SVM_Json')

        self.network.sink_Barchart_PNG =\
            self.network.create_sink('PNGFile', id_='Barchart_PNG')
        self.network.sink_Barchart_Tex =\
            self.network.create_sink('TexFile', id_='Barchart_Tex')

        self.network.sink_STest_CSV =\
            self.network.create_sink('CSVFile', id_='StatisticalTestFeatures_CSV')

        self.network.sink_Ranked_Percentages_Zip =\
            self.network.create_sink('ZipFile', id_='RankedPercentages_Zip')
        self.network.sink_Ranked_Percentages_CSV =\
            self.network.create_sink('CSVFile', id_='RankedPercentages_CSV')

        self.network.sink_Ranked_Posteriors_Zip =\
            self.network.create_sink('ZipFile', id_='RankedPosteriors_Zip')
        self.network.sink_Ranked_Posteriors_CSV =\
            self.network.create_sink('CSVFile', id_='RankedPosteriors_CSV')

        # Create links to sinks
        self.network.sink_ROC_PNG.input = self.network.node_ROC.outputs['output_png']
        self.network.sink_ROC_Tex.input = self.network.node_ROC.outputs['output_tex']
        self.network.sink_ROC_CSV.input = self.network.node_ROC.outputs['output_csv']

        self.network.sink_SVM_Json.input = self.network.node_SVM.outputs['output_json']

        self.network.sink_Barchart_PNG.input = self.network.node_Barchart.outputs['output_png']
        self.network.sink_Barchart_Tex.input = self.network.node_Barchart.outputs['output_tex']

        self.network.sink_STest_CSV.input = self.network.node_STest.outputs['performance']

        self.network.sink_Ranked_Percentages_Zip.input = self.network.node_Ranked_Percentages.outputs['output_zip']
        self.network.sink_Ranked_Percentages_CSV.input = self.network.node_Ranked_Percentages.outputs['output_csv']

        self.network.sink_Ranked_Posteriors_Zip.input = self.network.node_Ranked_Posteriors.outputs['output_zip']
        self.network.sink_Ranked_Posteriors_CSV.input = self.network.node_Ranked_Posteriors.outputs['output_csv']

        # Create two constant nodes
        self.network.node_Ranked_Percentages.inputs['scores'] = ['percentages']
        self.network.node_Ranked_Posteriors.inputs['scores'] = ['posteriors']

        # Create sources that are not in WORC and set them
        self.network.source_LabelType = self.network.create_source('String', id_='LabelType')
        self.network.source_Ensemble = self.network.create_source('String', id_='Ensemble')
        self.network.source_LabelType.input = [self.label_type]
        self.network.source_Ensemble.input = [self.ensemble]

        # Create sources if not supplied by a WORC network
        if self.mode == 'StandAlone':
            self.network.source_Estimator = self.network.create_source('HDF5', id_='Estimator')
            self.network.source_PatientInfo = self.network.create_source('PatientInfoFile', id_='PatientInfo')
            self.network.source_Images = self.network.create_source('ITKImageFile', id_='Images', nodegroup='patients')
            self.network.source_Segmentations = self.network.create_source('ITKImageFile', id_='Segmentations', nodegroup='patients')
            self.network.source_Config = self.network.create_source('ParameterFile', id_='Config')

            self.labels = list()
            self.network.source_Features = list()
            for idx in range(0, len(self.features)):
                label = 'Features_' + str(idx)
                self.labels.append(label)
                self.network.source_Features.append(self.network.create_source('HDF5', id_=label, nodegroup='features'))

        # Create links to sources that are not supplied by a WORC network
        self.network.node_ROC.inputs['ensemble'] = self.network.source_Ensemble.output
        self.network.node_ROC.inputs['label_type'] = self.network.source_LabelType.output

        self.network.node_SVM.inputs['ensemble'] = self.network.source_Ensemble.output
        self.network.node_SVM.inputs['label_type'] = self.network.source_LabelType.output

        self.network.node_Barchart.inputs['estimators'] = self.network.source_Ensemble.output
        self.network.node_Barchart.inputs['label_type'] = self.network.source_LabelType.output

        self.network.node_Ranked_Percentages.inputs['ensemble'] = self.network.source_Ensemble.output
        self.network.node_Ranked_Percentages.inputs['label_type'] = self.network.source_LabelType.output

        self.network.node_Ranked_Posteriors.inputs['ensemble'] = self.network.source_Ensemble.output
        self.network.node_Ranked_Posteriors.inputs['label_type'] = self.network.source_LabelType.output

        # Create links to the sources that could be in a WORC network
        if self.mode == 'StandAlone':
            # Sources from the Evaluate network are used
            self.network.node_ROC.inputs['prediction'] = self.network.source_Estimator.output
            self.network.node_ROC.inputs['pinfo'] = self.network.source_PatientInfo.output

            self.network.node_SVM.inputs['prediction'] = self.network.source_Estimator.output
            self.network.node_SVM.inputs['pinfo'] = self.network.source_PatientInfo.output

            self.network.node_Barchart.inputs['prediction'] = self.network.source_Estimator.output

            self.network.links_STest_Features = list()
            for idx, label in enumerate(self.labels):
                self.network.links_STest_Features.append(self.network.node_STest.inputs['features'][str(label)] << self.network.source_Features[idx].output)
                self.network.links_STest_Features[idx].collapse = 'features'
            self.network.node_STest.inputs['patientclass'] = self.network.source_PatientInfo.output
            self.network.node_STest.inputs['config'] = self.network.source_Config.output

            self.network.node_Ranked_Percentages.inputs['estimator'] = self.network.source_Estimator.output
            self.network.node_Ranked_Percentages.inputs['pinfo'] = self.network.source_PatientInfo.output
            self.network.link_images_perc = self.network.create_link(self.network.source_Images.output, self.network.node_Ranked_Percentages.inputs['images'])
            self.network.link_images_perc.collapse = 'patients'
            self.network.link_segmentations_perc = self.network.create_link(self.network.source_Segmentations.output, self.network.node_Ranked_Percentages.inputs['segmentations'])
            self.network.link_segmentations_perc.collapse = 'patients'

            self.network.node_Ranked_Posteriors.inputs['estimator'] = self.network.source_Estimator.output
            self.network.node_Ranked_Posteriors.inputs['pinfo'] = self.network.source_PatientInfo.output
            self.network.link_images_post = self.network.create_link(self.network.source_Images.output, self.network.node_Ranked_Posteriors.inputs['images'])
            self.network.link_images_post.collapse = 'patients'
            self.network.link_segmentations_post = self.network.create_link(self.network.source_Segmentations.output, self.network.node_Ranked_Posteriors.inputs['segmentations'])
            self.network.link_segmentations_post.collapse = 'patients'
        else:
            # Sources from the WORC network are used
            print 'WIP'

    def set(self, estimator=None, pinfo=None, images=None,
            segmentations=None, config=None, features=None,
            sink_data={}):
        '''
        Set the sources and sinks based on the provided attributes.
        '''
        if self.mode == 'StandAlone':
            self.source_data = dict()
            self.sink_data = dict()

            self.source_data['Estimator'] = estimator
            self.source_data['PatientInfo'] = pinfo
            self.source_data['Images'] = images
            self.source_data['Segmentations'] = segmentations
            self.source_data['Config'] = config
            self.source_data['LabelType'] = self.label_type
            self.source_data['Ensemble'] = self.ensemble

            for feature, label in zip(features, self.labels):
                self.source_data[label] = feature

            if 'ROC_PNG' not in sink_data.keys():
                self.sink_data['ROC_PNG'] = ("vfs://output/{}/ROC_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            if 'ROC_Tex' not in sink_data.keys():
                self.sink_data['ROC_Tex'] = ("vfs://output/{}/ROC_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            if 'ROC_CSV' not in sink_data.keys():
                self.sink_data['ROC_CSV'] = ("vfs://output/{}/ROC_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

            if 'SVM_Json' not in sink_data.keys():
                self.sink_data['SVM_Json'] = ("vfs://output/{}/performance_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

            if 'Barchart_PNG' not in sink_data.keys():
                self.sink_data['Barchart_PNG'] = ("vfs://output/{}/Barchart_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            if 'Barchart_Tex' not in sink_data.keys():
                self.sink_data['Barchart_Tex'] = ("vfs://output/{}/Barchart_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

            if 'StatisticalTestFeatures_CSV' not in sink_data.keys():
                self.sink_data['StatisticalTestFeatures_CSV'] = ("vfs://output/{}/StatisticalTestFeatures_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

            if 'RankedPercentages_Zip' not in sink_data.keys():
                self.sink_data['RankedPercentages_Zip'] = ("vfs://output/{}/RankedPercentages_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            if 'RankedPercentages_CSV' not in sink_data.keys():
                self.sink_data['RankedPercentages_CSV'] = ("vfs://output/{}/RankedPercentages_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

            if 'RankedPosteriors_Zip' not in sink_data.keys():
                self.sink_data['RankedPosteriors_Zip'] = ("vfs://output/{}/RankedPosteriors_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)
            if 'RankedPosteriors_CSV' not in sink_data.keys():
                self.sink_data['RankedPosteriors_CSV'] = ("vfs://output/{}/RankedPosteriors_{{sample_id}}_{{cardinality}}{{ext}}").format(self.name)

        else:
            print('[WORC Warning] Evaluate set attribute not needed when WORC network is provided!')

    def execute(self):
        """ Execute the network through the fastr.network.execute command. """
        # Draw and execute nwtwork
        self.network.draw_network(self.network.id, draw_dimension=True)
        self.network.execute(self.source_data, self.sink_data, execution_plugin=self.fastr_plugin, tmpdir=self.fastr_tmpdir)



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
from PREDICT.StatisticalTestFeatures import StatisticalTestFeatures


def main():
    parser = argparse.ArgumentParser(description='Radiomics classification')
    parser.add_argument('-feat', '--feat', metavar='features',
                        nargs='+', dest='feat', type=str, required=True,
                        help='Patient features input of first modality (HDF)')
    parser.add_argument('-pc', '--pc', metavar='Patientinfo',
                        dest='pc',
                        type=str, required=True, nargs='+',
                        help='Classification of patient')
    parser.add_argument('-cf', '--conf', metavar='config', nargs='+',
                        dest='cf', type=str, required=True,
                        help='Configuration')
    parser.add_argument('-perf', '--perf', metavar='performance',
                        dest='perf', type=str, required=True, nargs='+',
                        help='Performance (JSON)')
    args = parser.parse_args()

    StatisticalTestFeatures(features=args.feat,
                            patientinfo=args.pc,
                            config=args.cf,
                            output=args.perf,
                            verbose=False)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from collections import OrderedDict, MutableMapping
import re


class ElastixParameterFile(MutableMapping):
    """
    ElastixParameterFile is a dictionary object to allow access to parameter file in a style like::

    >>> params = ElastixParameterFile('/path/to/file/params.txt')
    >>> params[key].value
    >>> params[key].value = 'new_value'
    >>> params[linenr].value
    >>> params[key].comment
    >>> params[key].comment = 'This inserts/changes the comment on this line'
    >>> params[linenr].comment
    >>> params[key].linenr

    ElastixParameterFile is a MutableMapping, meaning it offers all functionality of a dict object.
    You can index both on setting key as well as line number. The returns values are ElastixParameterLine
    objects that give access to a key, value, comment and linenr. This allow easy manipulation of certain
    lines in the parameter file.
    """
    class ElastixParameterLine(list):
        def __init__(self, data):
            if len(data) != 4:
                raise ValueError(
                    'ElastixParameterLine must have 4 components!')

            if (not isinstance(data[0], int) or
                not (isinstance(data[1], (float, int, str, tuple, list)) or data[1] is None) or
                not (isinstance(data[2], str) or data[2] is None) or
                    not isinstance(data[3], str)):
                raise TypeError('ElastixParameterLine must be of types [int, list/None, str/None, str], found [%s, %s, %s, %s]' % (type(data[0]).__name__, type(data[1]).__name__, type(data[2]).__name__, type(data[3]).__name__))

            if data[1] is not None:
                if isinstance(data[1], (int, float, str)):
                    data[1] = [data[1]]

                data[1] = [self.parse_value(v) for v in data[1]]

            super(ElastixParameterFile.ElastixParameterLine, self).__init__(data)

        def __repr__(self):
            if self.value is None:
                key_val_str = ''
            elif len(self.value) > 10:
                key_val_str = '({} [...({:d} elem)...])'.format(self.key, len(self.value))
            else:
                value_str = ' '.join([self.unparse_value(v) for v in self.value])
                if len(value_str) > 15:
                    value_str = '{}...'.format(value_str[:15])

                key_val_str = '({key} {value})'.format(key=self.key, value=value_str)

            if self.comment is None:
                comment_str = ''
            elif len(self.comment) > 15:
                comment_str = '//{}...'.format(self.comment[:15])
            else:
                comment_str = '//{}'.format(self.comment)

            return '<ElastixParameterLine {nr}: {keyval}{comment}>'.format(nr=self.linenr, keyval=key_val_str, comment=comment_str)

        def __str__(self):
            if self.value is None:
                return self.comment_str
            elif self.comment is None:
                return self.key_value_str
            else:
                output = '{key_value} {comment}'.format(
                    key_value=self.key_value_str,
                    comment=self.comment_str)

                return output

        @property
        def linenr(self):
            return self[0]

        @property
        def value(self):
            return self[1]

        @value.setter
        def value(self, value):
            # Make sure value is a list
            if isinstance(value, (str, float, int)):
                value = [value]
            elif not isinstance(value, list):
                try:
                    value = list(value)
                except TypeError:
                    value = [value]

            self[1] = value

        @property
        def key_value_str(self):
            if self.value is None:
                return ''

            return '({key} {value})'.format(key=self.key, value=' '.join([self.unparse_value(v) for v in self.value]))

        @property
        def comment(self):
            return self[2]

        @comment.setter
        def comment(self, value):
            if not isinstance(value, str):
                raise TypeError('Comments have to be of str type!')

            if value[0] != ' ':
                value = ' ' + value

            self[2] = value

        @property
        def comment_str(self):
            if self.comment is None:
                return ''

            return '//{comment}'.format(comment=self.comment)

        @property
        def key(self):
            return self[3]

        @staticmethod
        def parse_value(item):
            """
            Parse a value in an elastix parameter file and cast it into the right
            type. Supported types are string, integer and float.
            """
            if isinstance(item, (int, float)):
                return item

            if item[0] == '"' and item[-1] == '"':
                item = item[1:-1]
            else:
                try:
                    item = int(item)
                except ValueError:
                    try:
                        item = float(item)
                    except ValueError:
                        item = str(item)

            return item

        @staticmethod
        def unparse_value(item):
            """
            Parse a python variable into the format used in an elastix parameter
            file. Supported types are string, integer and float.
            """
            if isinstance(item, str):
                item = '"%s"' % item
            else:
                item = str(item)

            return item

    """
    Create a ElastixParameterFile object. If filename is given the file is read
    and parsed into the object.
    """
    _re_line_parse = re.compile(r'\s*(\(.+\s+.*\))?\s*(//.*)?')
    _re_keyvalue_parse = re.compile(r'\((\S+)\s+(.*)\)')
    _re_value_parse = re.compile(r'(".*?"|-?\d+\.?\d*)')

    def __init__(self, filename=None):
        self._data = OrderedDict()

        if filename is not None:
            self.parse(filename)

    def parse(self, filename):
        """Load an elastrix parameter file and parse it into the ElastixParameterFile object.
        """
        self._data = OrderedDict()

        if filename is not None:
            with open(filename, 'r') as input_file:
                for linenr, line in enumerate(input_file):
                    match_obj = re.match(self._re_line_parse, line)

                    if match_obj.group(1) is None:
                        key = 'dummy_%d' % linenr
                        value = None
                    else:
                        submatch = re.match(self._re_keyvalue_parse, match_obj.group(1))
                        key = submatch.group(1)
                        value = re.findall(self._re_value_parse, submatch.group(2))

                    if match_obj.group(2) is None:
                        comment = None
                    else:
                        comment = match_obj.group(2)[2:]  # Strip of //

                    self._data[key] = self.ElastixParameterLine([linenr + 1, value, comment, key])

    def __getitem__(self, key):
        if isinstance(key, str):
            return self._data[key]
        elif isinstance(key, int):
            out = next((value for value in self._data.itervalues() if value[0] == key), None)
            if out is None:
                raise KeyError('Line not found in parameter file')

            return out
        else:
            raise TypeError('Key is of incorrect type!')

    def __setitem__(self, key, value):
        comment = None

        if not isinstance(key, str):
            key = self[key].key

        # If we get a ElastixParameterLine, insert as is
        if isinstance(value, self.ElastixParameterLine):
            # Unpack the desired fields
            comment = value.comment
            value = value.value

        # Check to update value or insert a new line
        if key in self._data:
            self._data[key].value = value
        else:
            if not isinstance(value, list):
                value = [value]
            line = len(self._data) + 1
            self._data[key] = self.ElastixParameterLine([line, value, comment, key])

    def __delitem__(self, key):
        if isinstance(key, str):
            del self._data[key]
        else:
            # Try to get a proper str key
            key = self[key].key
            del self._data[key]

    def __iter__(self):
        return self._data.__iter__()

    def __len__(self):
        return len(self._data)

    def __repr__(self):
        return '<ElastixParameterFile containing {:d} lines>'.format(len(self._data))

    def __str__(self):
        output = '\n'.join([str(v) for v in self._data.values()])

        return output

    def addline(self, key=None, value=None, comment=None):
        if key is None and value is not None:
            raise ValueError('Adding a value with a dummy key seems like a BAD idea!')

        if key is None:
            key = 'dummy_{}'.format(len(self))

        if key in self._data:
            raise ValueError('Cannot add a line with an already existing key!')

        linenr = len(self) + 1

        self._data[key] = self.ElastixParameterLine([linenr, value, comment, key])

    """
    Write the ElastrixParameterFile object back to an actual file on the disk.
    """
    def write(self, filename):
        with open(filename, 'w') as outfile:
            outfile.write(str(self))

if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--infile', '-i', metavar='input_transform.txt', required=True, help='the elastix parameter file to modify')
    parser.add_argument('--outfile', '-o', metavar='output_transform.txt', required=True, help='the resulting elastix parameter file')
    parser.add_argument('--set', '-s', metavar='key=val', nargs="*", help='the items to override in the parameter file')
    parser.add_argument('--verbose', '-v', action="store_true", help='use verbose output')

    args = parser.parse_args()

    input_file = args.infile
    output_file = args.outfile
    if args.set is not None:
        substitutions = OrderedDict([x.split('=', 1) for x in args.set])
    else:
        substitutions = OrderedDict()
    VERBOSE = args.verbose

    if VERBOSE:
        print('Loading input {}'.format(input_file))

    params = ElastixParameterFile(input_file)

    for key, value in substitutions.items():
        substitutions[key] = [params.ElastixParameterLine.parse_value(x) for x in value.split(',')]

    if VERBOSE:
        print('Substituting values')
        for key, value in substitutions.items():
            print('Set "{}" to {}'.format(key, value))

    params.update(substitutions)

    if VERBOSE:
        print('Writing result to {}'.format(output_file))

    params.write(output_file)



import argparse
import json
import re


def main():
    parser = argparse.ArgumentParser(description='Create a JSON from an transformix output points file')
    parser.add_argument('--in', metavar='IN', dest='input', type=str, required=True, help='The output points to parse')
    args = parser.parse_args()

    output = {'inputindex': {},
              'inputpoint': {},
              'outputindex': {},
              'outputpoint': {},
              'deformation': {}}

    with open(args.input) as fin:
        for line in fin:
            match = re.match('Point\s+(?P<id>\d+)\s+; InputIndex = \[(?P<inindex>[\s\d]+)\]\s+; InputPoint = \[(?P<inpoint>[\s\d\.-]+)\]\s+; OutputIndexFixed = \[(?P<outindex>[-\s\d]+)\]\s+; OutputPoint = \[(?P<outpoint>[\s\d\.-]+)\]\s+; Deformation = \[(?P<deform>[\s\d\.-]+)\]', line)
            if match:
                sample_id = match.group('id')
                output['inputindex'][sample_id] = [int(x) for x in match.group('inindex').split()]
                output['inputpoint'][sample_id] = [float(x) for x in match.group('inpoint').split()]
                output['outputindex'][sample_id] = [int(x) for x in match.group('outindex').split()]
                output['outputpoint'][sample_id] = [float(x) for x in match.group('outpoint').split()]
                output['deformation'][sample_id] = [float(x) for x in match.group('deform').split()]
            else:
                raise ValueError('Could not parse line: {}'.format(line))

    print('__INPUTINDEX__ = {}'.format(json.dumps(output['inputindex'])))
    print('__INPUTPOINT__ = {}'.format(json.dumps(output['inputpoint'])))
    print('__OUTPUTINDEX__ = {}'.format(json.dumps(output['outputindex'])))
    print('__OUTPUTPOINT__ = {}'.format(json.dumps(output['outputpoint'])))
    print('__DEFORMATION__ = {}'.format(json.dumps(output['deformation'])))


if __name__ == '__main__':
    main()



# Copyright 2011-2014 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import fastr
from fastr.data import url
from fastr.core.version import Version
from fastr.datatypes import URLType


class DicomImageDirectory(URLType):
    description = 'Dicom Image directory'
    extension = ''

    def _validate(self):
        value = self.value

        if url.isurl(self.value):
            value = url.get_path_from_url(value)

        try:
            if not os.path.isdir(value):
                return False

            contents = os.listdir(value)
            return any(x.endswith('.dcm') for x in contents)
        except ValueError:
            return False

    def action(self, name):
        if name is None:
            pass
        elif name == "ensure":
            if url.isurl(self.value):
                dir_name = url.get_path_from_url(self.value)
            else:
                dir_name = self.value

            fastr.log.debug('ensuring {} exists.'.format(dir_name))
            if not os.path.exists(dir_name):
                os.makedirs(dir_name)
        else:
            fastr.log.warning("unknown action")



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import SimpleITK as sitk


def main():
    parser = argparse.ArgumentParser(description='Image conversion')
    parser.add_argument('-in', '--in', metavar='input', nargs='+',
                        dest='im', type=str, required=False,
                        help='Image in (ITK Image)')
    parser.add_argument('-out', '--out', metavar='output',
                        dest='out', type=str, required=False,
                        help='Image out (ITK Image)')
    args = parser.parse_args()

    if type(args.im) is list:
        args.im = ''.join(args.im)

    if type(args.out) is list:
        args.out = ''.join(args.out)

    image = sitk.ReadImage(args.im)
    sitk.WriteImage(image, args.out)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
from segmentix import segmentix


def main():
    parser = argparse.ArgumentParser(description='Feature extraction')
    parser.add_argument('-im', '--im', metavar='image', nargs='+',
                        dest='im', type=str, required=False,
                        help='Images to calculate features on')
    parser.add_argument('-md', '--md', metavar='metadata', dest='md',
                        type=str, required=False, nargs='+',
                        help='Clinical data on patient (DICOM)')
    parser.add_argument('-segin', '--segin', metavar='segmentation', dest='seg',
                        type=str, required=False, nargs='+',
                        help='Segmentation input (ITK Image)')
    parser.add_argument('-mask', '--mask', metavar='mask', dest='mask',
                        type=str, required=False, nargs='+',
                        help='Mask (ITK Image)')
    parser.add_argument('-para', '--para', metavar='Parameters', nargs='+',
                        dest='para', type=str, required=True,
                        help='Parameters')
    parser.add_argument('-segout', '--segout', metavar='Features',
                        dest='out', type=str, required=False,
                        help='Segmentation output (ITK Image)')
    args = parser.parse_args()

    segmentix(image=args.im, segmentation=args.seg, parameters=args.para,
              output=args.out, metadata_file=args.md, mask=args.mask)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from skimage.measure import label, regionprops
import numpy as np


def ExtractNLargestBlobsn(binaryImage, numberToExtract=1):
    """Extract N largest blobs from binary image.

    Arguments:
        binaryImage: boolean numpy array one or several contours.
        numberToExtract: number of blobs to extract (integer).

    Returns:
        binaryImage: boolean numpy are containing only the N
                     extracted blobs.
    """

    # Get all the blob properties.
    labeledImage = label(binaryImage, connectivity=3)
    blobMeasurements = regionprops(labeledImage)

    if len(blobMeasurements) == 1:
        # Single blob, return input
        binaryImage = binaryImage
    else:
        # Get all the areas
        allAreas = list()
        allCoords = list()
        for blob in blobMeasurements:
            allAreas.append(blob.area)
            allCoords.append(blob.coords)

        allAreas = np.asarray(allAreas)
        if numberToExtract > 0:
            # For positive numbers, sort in order of largest to smallest.
            # Sort them.
            indices = np.argsort(-allAreas)
        elif numberToExtract < 0:
            # For negative numbers, sort in order of smallest to largest.
            # Sort them.
            indices = np.argsort(allAreas)
        else:
            raise ValueError("Number of blobs to extract should not be zero!")

        binaryImage = np.zeros(binaryImage.shape)
        # NOTE: There must be a more efficient way to do this
        for nblob in range(0, numberToExtract):
            nblob = abs(nblob)
            coords = allCoords[indices[nblob]]
            for coord in coords:
                binaryImage[coord[0], coord[1], coord[2]] = 1

    return binaryImage



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PREDICT.plotting.plot_SVM import plot_SVM
import argparse
import json


def main():
    parser = argparse.ArgumentParser(description='Plot the SVM Curve of an estimator')
    parser.add_argument('-prediction', '--prediction', metavar='prediction',
                        nargs='+', dest='prediction', type=str, required=True,
                        help='Prediction file (HDF)')
    parser.add_argument('-pinfo', '--pinfo', metavar='pinfo',
                        nargs='+', dest='pinfo', type=str, required=True,
                        help='Patient Info File (txt)')
    parser.add_argument('-ensemble', '--ensemble', metavar='ensemble',
                        nargs='+', dest='ensemble', type=str, required=True,
                        help='Length of ensemble (int)')
    parser.add_argument('-label_type', '--label_type', metavar='label_type',
                        nargs='+', dest='label_type', type=str, required=True,
                        help='Label name that is predicted (string)')
    parser.add_argument('-output_json', '--output_json', metavar='output_json',
                        nargs='+', dest='output_json', type=str, required=False,
                        help='File to write output to (json)')
    args = parser.parse_args()

    # Convert inputs to strings
    if type(args.prediction) is list:
        args.prediction = ''.join(args.prediction)

    if type(args.pinfo) is list:
        args.pinfo = ''.join(args.pinfo)

    if type(args.ensemble) is list:
        args.ensemble = int(args.ensemble[0])
        # ensemble = ''.join(ensemble)

    if type(args.output_json) is list:
        args.output_json = ''.join(args.output_json)

    if type(args.label_type) is list:
        args.label_type = ''.join(args.label_type)

    # Plot the statistics
    stats = plot_SVM(prediction=args.prediction,
                     label_data=args.pinfo,
                     ensemble=args.ensemble,
                     label_type=args.label_type)

    with open(args.output_json, 'w') as fp:
        json.dump(stats, fp, indent=4)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
@author: Zhenwei.Shi, Maastro

usage:

    paste this file in same directory as your main script file
    and use the following lines

    import DCM_ImgRT_Reader_ZW

    path1=r'c:\ABC\FolderContainingDicomOfimageCTscan'
    path2=r'c:\ABC\FolderContainingDicomOfRTstruct'
    DCM_ImgRT_Reader_ZW.Img_Bimask(image_path=path1,rtstruct_path=path2,ROI_name)

"""

import dicom as pydicom
import os
import numpy as np
from skimage import draw
import SimpleITK as sitk
import re
import glob
import natsort


def get_dicoms(input_dir, logfile=None):
    dicom_files = glob(input_dir + '/*.dcm')
    dicom_files = natsort.natsorted(dicom_files, alg=ns.IGNORECASE)

    dicoms = list()
    for i_dicom in dicom_files:
        dicoms.append(pydicom.read_file(i_dicom))

    return dicoms


def load_dicom(dicom_folder):
    dicom_reader = sitk.ImageSeriesReader()
    dicom_file_names = dicom_reader.GetGDCMSeriesFileNames(dicom_folder)
    dicom_reader.SetFileNames(dicom_file_names)
    dicom_image = dicom_reader.Execute()

    return dicom_image


def poly2mask(vertex_row_coords, vertex_col_coords, shape):
    fill_row_coords, fill_col_coords = draw.polygon(vertex_row_coords, vertex_col_coords, shape)
    mask = np.zeros(shape, dtype=np.bool)
    mask[fill_row_coords, fill_col_coords] = True
    return mask


def get_pixels(scan):
    image = np.stack([s.pixel_array for s in scan])
    image = image.astype(np.int16)
#    image[image == -2000] = 0
    intercept = scan[0].RescaleIntercept
    slope = scan[0].RescaleSlope
    if slope != 1:
        image = slope * image.astype(np.float64)
        image = image.astype(np.int16)
    image += np.int16(intercept)
    return np.array(image, dtype=np.int16)


def ROI_ref(rtstruct_path, ROI_name):
    mask_vol = load_dicom(rtstruct_path)
    M = mask_vol[0]
    for i in range(len(M.StructureSetROISequence)):
        if str(ROI_name) == M.StructureSetROISequence[i].ROIName:
            ROI_number = M.StructureSetROISequence[i].ROINumber
            break
    return int(ROI_number)


def match_ROIid(rtstruct_path, ROI_number):
    mask_vol = load_dicom(rtstruct_path)
    M = mask_vol[0]
    for j in range(len(M.StructureSetROISequence)):
		if ROI_number == M.ROIContourSequence[j].RefdROINumber:
			break
    return j


def RTStructReader(RTStructFile, ROI_name, image_folder):
    '''
    Input:
        image_path is directory containing dicom files
        rtstruct_path is directory containing RTstruct files
		ROI_name is the ROI stored in RTstructure

    Output:
        mask file written in current directory
        image volume file written in current directory

    '''
	# Volume of DICOM files and RTstruct files
    image_vol = load_dicom(image_folder)
    L = pydicom.read_file(glob.glob(image_folder + '/*.dcm')[0])
    mask_vol = pydicom.read_file(RTStructFile)
    print dir(mask_vol)

	# Slices all have the same basic information including slice size, patient position, etc.
    # L = image_vol[0]
    print image_vol
    # LP = get_pixels(image_vol)
    LP = image_vol
    num_slice = LP.GetSize()[2]

    mask = np.zeros([num_slice, L.Rows, L.Columns],dtype=np.uint8)

    xres=np.array(L.PixelSpacing[0])
    yres=np.array(L.PixelSpacing[1])

    # ROI_number = ROI_ref(rtstruct_path,ROI_name)
    # Check the number of the ROI corresponding to the ROI name to extract
    for i in range(len(mask_vol.StructureSetROISequence)):
        if str(ROI_name) == mask_vol.StructureSetROISequence[i].ROIName:
            ROI_number = mask_vol.StructureSetROISequence[i].ROINumber
            break

    ROI_id = match_ROIid(rtstruct_path,ROI_number)

    for k in range(len(M.ROIContourSequence[ROI_id].ContourSequence)):
        UIDrt=''

        UIDrt = M.ROIContourSequence[ROI_id].ContourSequence[k].ContourImageSequence[0].ReferencedSOPInstanceUID

        for i in range(num_slice):
            UIDslice = image_vol[i].SOPInstanceUID

            if UIDrt==UIDslice:
                sliceOK = i
                break

        x=[]
        y=[]
        z=[]

        m=M.ROIContourSequence[ROI_id].ContourSequence[k].ContourData

        for i in range(0,len(m),3):
            x.append(m[i+1])
            y.append(m[i+0])
            z.append(m[i+2])

        x=np.array(x)
        y=np.array(y)
        z=np.array(z)

        x-= L.ImagePositionPatient[1]
        y-= L.ImagePositionPatient[0]
        z-= L.ImagePositionPatient[2]

        pts = np.zeros([len(x),3])
        pts[:,0] = x
        pts[:,1] = y
        pts[:,2] = z
        a=0
        b=1
        p1 = xres;
        p2 = yres;

        # What is this exactly: Orientation times voxels spacing in a 2x2 matrix?
        m=np.zeros([2,2])
        m[0,0]=image_vol[sliceOK].ImageOrientationPatient[a]*p1
        m[0,1]=image_vol[sliceOK].ImageOrientationPatient[a+3]*p2
        m[1,0]=image_vol[sliceOK].ImageOrientationPatient[b]*p1
        m[1,1]=image_vol[sliceOK].ImageOrientationPatient[b+3]*p2

        # Transform points from reference frame to image coordinates
        m_inv=np.linalg.inv(m)
        pts = (np.matmul((m_inv),(pts[:,[a,b]]).T)).T

        # So you create the mask by making a polygon: how to treat boundary pixels? How was the original contour made?
        mask[sliceOK,:,:] = np.logical_or(mask[sliceOK,:,:],poly2mask(pts[:,0],pts[:,1],[LP.shape[1],LP.shape[2]]))

        LP-=np.min(LP)
        LP=LP.astype(np.float32)
        LP/=np.max(LP)
        LP*=255

        image=sitk.GetImageFromArray(LP.astype(np.float32))
        Mask=sitk.GetImageFromArray(mask)

    return image, Mask


if __name__ == '__main__':
    folder = 'HN1004_20170922_' #interobs05_20170910_CT
    RTStructFile = glob.glob('/home/martijn/Documents/test/' + folder + '/SCANS/DCM/DICOM/*.dcm')[0]
    # image_files = glob.glob('/home/martijn/Documents/test/' + folder + '/SCANS/DCM/DICOM/*.dcm')
    image_folder = '/home/martijn/Documents/test/' + folder + '/SCANS/DCM/DICOM'
    name = 'GTV-1auto(DR)'
    RTStructReader(RTStructFile, name, image_folder)



#!/usr/bin/env python

# Copyright 2017-2018 Biomedical Imaging Group Rotterdam, Departments of
# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PREDICT.plotting.plot_barchart import plot_barchart
import argparse


def main():
    parser = argparse.ArgumentParser(description='Plot the barchart of the hyperparameters of a number of hyperparameter settings')
    parser.add_argument('-prediction', '--prediction', metavar='prediction',
                        nargs='+', dest='prediction', type=str, required=True,
                        help='Prediction file (HDF)')
    parser.add_argument('-estimators', '--estimators', metavar='estimators',
                        nargs='+', dest='estimators', type=str, required=True,
                        help='Number of estimators (int)')
    parser.add_argument('-label_type', '--label_type', metavar='label_type',
                        nargs='+', dest='label_type', type=str, required=True,
                        help='Label name that is predicted (string)')
    parser.add_argument('-output_png', '--output_png', metavar='output_png',
                        nargs='+', dest='output_png', type=str, required=False,
                        help='File to write output to (PNG)')
    parser.add_argument('-output_tex', '--output_tex', metavar='output_tex',
                        nargs='+', dest='output_tex', type=str, required=False,
                        help='File to write output to (tex)')
    args = parser.parse_args()

    # Convert inputs from lists to elements for single inputs
    if type(args.prediction) is list:
        args.prediction = ''.join(args.prediction)

    if type(args.label_type) is list:
        args.label_type = ''.join(args.label_type)

    if type(args.output_png) is list:
        args.output_png = ''.join(args.output_png)

    if type(args.output_tex) is list:
        args.output_tex = ''.join(args.output_tex)

    # Number of estimators should be an integer
    if type(args.estimators) is list:
        args.estimators = int(args.estimators[0])

    # Plot the barchart
    plot_barchart(prediction=args.prediction,
                  estimators=args.estimators,
                  label_type=args.label_type,
                  output_tex=args.output_tex,
                  output_png=args.output_png)


if __name__ == '__main__':
    main()
